#!/bin/ksh
# Licensed Materials - Property of IBM
# 5724-S67
# (c) Copyright IBM Corp. 1998, 2010   All Rights Reserved.
# US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
export LC_TIME="C"

ACTUATOR_LOG=../log/$PROPSFILE.log
TEMPFILE=$TEMPFILE5
SHARED_SCRIPT=../bin/getguardium_shared.sh
ERRORFILE=$PROPSFILE.err
LOGTYPE="LOGIN"

##################### INTERNAL FUNCTIONS #######################
function printlog {
 echo `date "+%a %D %T"` $1 >> $ACTUATOR_LOG
}

function increaseerr {
 NUM=`cat $ERRORFILE` && ((NUM += 1)) && echo $NUM > $ERRORFILE
}

# Here will check the errors occured during collect
function finalize_collect {
 printlog "========================================================"
 ERRNUM=`cat $ERRORFILE`
 rm -f $ERRORFILE
 if [ ERRNUM -eq 0 ] ; then
  rotateprops
  printlog "No errors during collect: COMPLETE SUCCESS"
  exit 0
 else
  # We have an error file, now we will check if we have complete broken collect
  if [ ERRNUM -lt 5 ] ; then 
   printlog "$ERRNUM errors occured during collect: PARTIAL SUCCESS"
   rotateprops
   exit 0
  else
   printlog "All sublogs collect failed of skipped: COMPLETE FAILURE"
   rotateprops
   exit 1
  fi
 fi
}

# Rename temporaty props file
function rotateprops {
 mv -f $PROPSFILE.tmp $PROPSFILE
 if [[ $? -ne 0 ]] ; then
  printlog "ERROR: temporaty props file rename failed!"
  exit 1
 fi
}
##################### INTERNAL FUNCTIONS #######################

# Check log template
if [ "a$OPTION7" = "a" ] ; then 
 printlog "WARNING: Guardium login attempts tracking log filename template is not defined. Skipping."
 increaseerr
 > $TEMPFILE
 finalize_collect
fi

# Now we start with 5th sublog processing
printlog "Processing Guardium login attempts tracking sublog (#5)"
$SHARED_SCRIPT $TEMPFILE $OPTION7 $LOGTYPE $OPTION2 $PROPSFILE

if [[ $? -eq 0 ]] ; then
 printlog "Guardium login attempts tracking sublog collect: SUCCESS"
 finalize_collect
else
 printlog "ERROR: Guardium login attempts tracking sublog collect: FAILURE"
 increaseerr
 finalize_collect
fi

printlog "Unknown error occured. Exiting."
exit 1
