REM Licensed Materials - Property of IBM
REM 5724-S67
REM (c) Copyright IBM Corp. 1998, 2010   All Rights Reserved.
REM US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
@echo off
rem ############################################################################
rem      MAIN BODY
rem ############################################################################
setlocal
set TEMP_FILE=%~1
set INF_LOG_DIR=%~2
set INST_NAME=%~3
for %%A in (%*) do set PROPS_FILE=%%~A
set LOG_FILE="..\log\%PROPS_FILE%.log"

echo [%date%][%time%] Processing started ...> %LOG_FILE%

if "%TEMP_FILE%" EQU "" (
  echo Failure: The name of output file is not specified.>> %LOG_FILE%
  echo [%date%][%time%] Processing finished [FAILURE].>> %LOG_FILE%
  endlocal 
  exit /b 1
)
if "%INF_LOG_DIR%" EQU "" (
  echo Failure: The Informix Log Directory is not specified.>> %LOG_FILE%
  echo [%date%][%time%] Processing finished [FAILURE].>> %LOG_FILE%
  endlocal
  exit /b 2
)
if "%INST_NAME%" EQU "" (
  echo Failure: The Informix Server Name is not specified.>> %LOG_FILE%
  echo [%date%][%time%] Processing finished [FAILURE].>> %LOG_FILE%
  endlocal
  exit /b 3
)
if not exist "%INF_LOG_DIR%" (
  echo Failure: Do not have read access on specified Informix Log Directory, "%INF_LOG_DIR%".>> %LOG_FILE%
  echo [%date%][%time%] Processing finished [FAILURE].>> %LOG_FILE%
  endlocal
  exit /b 4
)
if not exist "..\bin\date.exe" (
  echo Failure: Could not find the crucial auxiliary binary, "..\bin\date.exe".>> %LOG_FILE%
  echo [%date%][%time%] Processing finished [FAILURE].>> %LOG_FILE%
  endlocal
  exit /b 5
)
if not exist "..\bin\smalldd.exe" (
  echo Failure: Could not find the crucial auxiliary binary, "..\bin\smalldd.exe".>> %LOG_FILE%
  echo [%date%][%time%] Processing finished [FAILURE].>> %LOG_FILE%
  endlocal
  exit /b 6
)

set LOG_FAILURE=NO
type nul>"%TEMP_FILE%"

set skip=
>"%PROPS_FILE%_aux3.vbs" findstr "'%skip%VBS_3_INFORMIX" "%~f0"

if not exist "%PROPS_FILE%" (
  call :do_initial_collection
) else (
  call :do_non_initial_collect
)

if "%LOG_FAILURE%" == "YES" (
  if exist "%TEMP_FILE%" ( del /F/Q "%TEMP_FILE%" )
  echo [%date%][%time%] Processing finished [FAILURE].>> %LOG_FILE%
  endlocal
  exit /b 7 
)

if exist "%TEMP_FILE%.small"         ( del /F/Q "%TEMP_FILE%.small"         )
if exist "%PROPS_FILE%_aux.vbs"      ( del /F/Q "%PROPS_FILE%_aux.vbs"      )
if exist "%PROPS_FILE%_aux2.vbs"     ( del /F/Q "%PROPS_FILE%_aux2.vbs"     )
if exist "%PROPS_FILE%_aux3.vbs"     ( del /F/Q "%PROPS_FILE%_aux3.vbs"     )
if exist "%PROPS_FILE%.files_info"   ( del /F/Q "%PROPS_FILE%.files_info"   )
if exist "%PROPS_FILE%.actual_files" ( del /F/Q "%PROPS_FILE%.actual_files" )

echo [%date%][%time%] Processing finished [SUCCESS]. >> %LOG_FILE%
endlocal
exit /b 0


rem ############################################################################
rem      :do_initial_collection - performs initial data collection,  collects all files in %INF_LOG_DIR% directory, which fit to the naming 
rem         convention for Informix audit log files (%INST_NAME%.integer) and which were modified during the last elapsed 2 days.
rem ############################################################################
:do_initial_collection
echo [%date%][%time%] Going to perform initial collect ("%PROPS_FILE%" does not exist).>> %LOG_FILE%
for /F "delims=" %%I IN ('..\bin\date.exe +%%Y%%m%%d%%H%%M%%S -d "2 days ago"') DO (set START_TIME=%%I)
call :get_all_relevant_log_files
call :collect_actual_log_files %START_TIME%
goto :EOF


rem ############################################################################
rem      :do_non_initial_collect - performs non-initial collect using previously produced state information.
rem ############################################################################
:do_non_initial_collect
set PREV_MAX_TIMESTAMP=
FOR /F "tokens=1-2* delims==" %%a IN ('findstr MAX_TIMESTAMP= "%PROPS_FILE%"') DO (set PREV_MAX_TIMESTAMP=%%b)
if "%PREV_MAX_TIMESTAMP%" EQU "" (
  call :do_non_initial_collect_old
) else (
  call :do_non_initial_collect_new %PREV_MAX_TIMESTAMP%
)
goto :EOF


rem ############################################################################
rem      :do_non_initial_collect_new - performs non-initial collect using state information produced by the current version of this script.
rem ############################################################################
:do_non_initial_collect_new
echo [%date%][%time%] Going to perform non initial collect starting from %PREV_MAX_TIMESTAMP%.>> %LOG_FILE%
call :get_all_relevant_log_files
call :collect_actual_log_files %1
goto :EOF


rem ############################################################################
rem      :get_all_relevant_log_files - stores names of files from %INF_LOG_DIR% directory, which fit to the naming 
rem         convention for Informix audit log files (%INST_NAME%.integer), to "%PROPS_FILE%.actual_files".
rem ############################################################################
:get_all_relevant_log_files
if exist "%PROPS_FILE%.actual_files" ( del /F/Q "%PROPS_FILE%.actual_files" )
for /F "delims=" %%i in ('dir /A-D /B /OD "%INF_LOG_DIR%"') do (
  @echo %%i | findstr /R "\<%INST_NAME%.[0-9][0-9]*\>">nul
  if NOT errorlevel 1 (
    echo %%i>>"%PROPS_FILE%.actual_files"
  )
)
goto :EOF


rem ############################################################################
rem      :collect_actual_log_files - generic function that performs actual data collection.
rem                %1 - The timestamp (e.g. "20100311154526") for log files selection.
rem ############################################################################
:collect_actual_log_files
if not exist "%PROPS_FILE%.actual_files" (
  call :nothing_to_collect %1
) else (
  call :collect_actual_log_files_aux %1
)
goto :EOF


rem ############################################################################
rem      :nothing_to_collect - performs collect when there is no data to collect.
rem ############################################################################
:nothing_to_collect
echo [%date%][%time%] Nothing to collect.>> %LOG_FILE%
if not exist "%PROPS_FILE%" (
  echo MAX_TIMESTAMP=%1>"%PROPS_FILE%"
  echo _domainname=%USERDOMAIN%>>"%PROPS_FILE%"
)
goto :EOF


rem ############################################################################
rem      :collect_actual_log_files_aux - performs collect when there are some files, which probably need to be collected.
rem                %1 - The timestamp (e.g. "20100311154526") for log files selection.
rem ############################################################################
:collect_actual_log_files_aux
set skip=
>"%PROPS_FILE%_aux.vbs" findstr "'%skip%VBS_INFORMIX" "%~f0"

if exist "%PROPS_FILE%.files_info" ( del /F/Q "%PROPS_FILE%.files_info" )
@cscript.exe //NoLogo "%PROPS_FILE%_aux.vbs" "%INF_LOG_DIR%" "%PROPS_FILE%.actual_files" %1>>"%PROPS_FILE%.files_info"

if not exist "%PROPS_FILE%.files_info" ( call :nothing_to_collect %1 && goto :EOF )

set /A LOG_FILES_NUMBER=0
for /F "tokens=1,2 delims=;" %%i in ('type "%PROPS_FILE%.files_info"') do (
  call :collect_log_file %%i %%j
  set MAX_TIMESTAMP=%%j
  set /A LOG_FILES_NUMBER+=1
)
if "%LOG_FAILURE%" == "YES" ( goto :EOF  )
if %LOG_FILES_NUMBER% EQU 0 ( call :nothing_to_collect %1 && goto :EOF )

echo MAX_TIMESTAMP=%MAX_TIMESTAMP%>"%PROPS_FILE%"
echo _domainname=%USERDOMAIN%>>"%PROPS_FILE%"
for /F "tokens=1,2 delims=;" %%i in ('findstr %MAX_TIMESTAMP% "%PROPS_FILE%.files_info"') do (
  call :remember_log_file %%i
)

goto :EOF


rem ############################################################################
rem      :collect_log_file - routine for collecting the given log file (%1) with the given file modification timestamp (%2).
rem ############################################################################
:collect_log_file
if "%LOG_FAILURE%" == "YES" ( goto :EOF  )

if not exist "%PROPS_FILE%" (
  call :collect_complete_log_file %1 %2
  goto :EOF
)

set REMEMBERED_FIRST_LINE=
FOR /F "tokens=1-2* delims==" %%a IN ('findstr FILE_FIRST_LINE_%1= "%PROPS_FILE%"') DO (set REMEMBERED_FIRST_LINE=%%~b)
if "%REMEMBERED_FIRST_LINE%" == "" (
  call :collect_complete_log_file %1 %2
  goto :EOF
)

set /A REMEMBERED_FILE_SIZE=0
FOR /F "tokens=1-2* delims==" %%a IN ('findstr FILE_SIZE_%1= "%PROPS_FILE%"') DO (set /A REMEMBERED_FILE_SIZE=%%b)
if "%REMEMBERED_FILE_SIZE%" == "" (
  call :collect_complete_log_file %1 %2
  goto :EOF
)

set FILE_FIRST_LINE=
for /F "delims=" %%I IN ('@cscript.exe //NoLogo "%PROPS_FILE%_aux3.vbs" "%INF_LOG_DIR%\%1"') DO (set FILE_FIRST_LINE=%%I)

if not "%FILE_FIRST_LINE%" == "%REMEMBERED_FIRST_LINE%" (
  call :collect_complete_log_file %1 %2
  goto :EOF
)

set /A CURRENT_FILE_SIZE=0
for %%i in ("%INF_LOG_DIR%\%1") do (set /A CURRENT_FILE_SIZE=%%~zi)

if "%CURRENT_FILE_SIZE%" == "%REMEMBERED_FILE_SIZE%" (
  echo [%date%][%time%]   "%INF_LOG_DIR%\%1" has not been changed since the previous collect, skipping.>>%LOG_FILE%
  goto :EOF
)

if %CURRENT_FILE_SIZE% LSS %REMEMBERED_FILE_SIZE% (
  call :collect_complete_log_file %1 %2
  goto :EOF
)

echo [%date%][%time%]   Collecting "%INF_LOG_DIR%\%1", file modification timestamp: %2, skipping first %REMEMBERED_FILE_SIZE% bytes.>>%LOG_FILE%
set /A REMEMBERED_FILE_SIZE+=1
call "..\bin\smalldd.exe" "%REMEMBERED_FILE_SIZE%" "EOF" "%INF_LOG_DIR%\%1" "%TEMP_FILE%.small"
if errorlevel 1 (
  echo [%date%][%time%]   Failed to collect "%INF_LOG_DIR%\%1".>>%LOG_FILE%
  set LOG_FAILURE=YES
  goto :EOF
)

for %%i in ("%TEMP_FILE%.small") do (
  set /A COLLECTED_FILE_SIZE_%1=%REMEMBERED_FILE_SIZE%+%%~zi-1
)

copy /Y /B "%TEMP_FILE%"+"%TEMP_FILE%.small" "%TEMP_FILE%">nul /B 2>>%LOG_FILE%
if errorlevel 1 (
  echo [%date%][%time%]   Failed to collect "%INF_LOG_DIR%\%1".>>%LOG_FILE%
  set LOG_FAILURE=YES
  goto :EOF
)
goto :EOF


rem ############################################################################
rem      :collect_log_file - routine for blind collecting the whole given log file (%1) with the given file modification timestamp (%2).
rem ############################################################################
:collect_complete_log_file
echo [%date%][%time%]   Collecting "%INF_LOG_DIR%\%1", file modification timestamp: %2.>>%LOG_FILE%
for %%i in ("%TEMP_FILE%") do (
  set /A COLLECTED_FILE_SIZE=%%~zi
)

copy /Y /B "%TEMP_FILE%"+"%INF_LOG_DIR%\%1" "%TEMP_FILE%">nul /B 2>>%LOG_FILE%
if errorlevel 1 (
  echo [%date%][%time%]   Failed to collect "%INF_LOG_DIR%\%1".>>%LOG_FILE%
  set LOG_FAILURE=YES
  goto :EOF
)

for %%i in ("%TEMP_FILE%") do (
  set /A COLLECTED_FILE_SIZE_%1=%%~zi-%COLLECTED_FILE_SIZE%
)

goto :EOF


rem ############################################################################
rem      :remember_log_file - routine for remembering state information related to the given log file (%1) for the next collect
rem ############################################################################
:remember_log_file
set /A LOG_FILE_SIZE=0
for %%i in ("%INF_LOG_DIR%\%1") do (
  set /A LOG_FILE_SIZE=%%~zi
)
if %LOG_FILE_SIZE% EQU 0 ( goto :EOF )

set VAL_NAME=COLLECTED_FILE_SIZE_%1
call set SAVED_SIZE=%%%VAL_NAME%%%

if NOT "%SAVED_SIZE%" == "" (
  set /A LOG_FILE_SIZE=%SAVED_SIZE%
)


set FILE_FIRST_LINE=
for /F "delims=" %%I IN ('@cscript.exe //NoLogo "%PROPS_FILE%_aux3.vbs" "%INF_LOG_DIR%\%1"') DO (set FILE_FIRST_LINE=%%I)

echo FILE_FIRST_LINE_%1="%FILE_FIRST_LINE%">>"%PROPS_FILE%"
echo FILE_SIZE_%1=%LOG_FILE_SIZE%>>"%PROPS_FILE%"
goto :EOF


rem ############################################################################
rem      :do_non_initial_collect_old - performs non-initial collect using state information produced by the previous version of this script.
rem ############################################################################
:do_non_initial_collect_old
echo [%date%][%time%] Going to perform non initial collect using state information produced by the previous version of this script.>>%LOG_FILE%

set REMEMBERED_FIRST_LINE=
FOR /F "tokens=1-2* delims= " %%a IN ('findstr firstLine "%PROPS_FILE%"') DO (set REMEMBERED_FIRST_LINE=%%c)

if "%REMEMBERED_FIRST_LINE%" == "" (
  call :get_all_relevant_log_files
  call :collect_actual_log_files 19000000000000
  goto :EOF
)

set /A REMEMBERED_FILE_SIZE=0
FOR /F "tokens=1-2* delims= " %%a IN ('findstr filePosition "%PROPS_FILE%"') DO (set /A REMEMBERED_FILE_SIZE=%%c)

set REMEMBERED_FILE_NAME=
for /F "delims=" %%i in ('dir /A-D /B /O-D "%INF_LOG_DIR%"') do (
  @echo %%i | findstr /R "\<%INST_NAME%.[0-9][0-9]*\>">nul
  if NOT errorlevel 1 (
    call :is_it_remembered_log_file %%i "%REMEMBERED_FIRST_LINE%"
    if errorlevel 1 ( 
      set REMEMBERED_FILE_NAME=%%i
      goto :remembered_log_file_is_found
    )
  )
)

call :get_all_relevant_log_files
call :collect_actual_log_files 19000000000000
goto :EOF

:remembered_log_file_is_found
set skip=
>"%PROPS_FILE%_aux2.vbs" findstr "'%skip%VBS_2_INFORMIX" "%~f0"

set MAX_TIMESTAMP=19000000000000
for /F "delims=" %%I IN ('@cscript.exe //NoLogo "%PROPS_FILE%_aux2.vbs" "%INF_LOG_DIR%\%REMEMBERED_FILE_NAME%"') DO (set MAX_TIMESTAMP=%%I)

set FILE_FIRST_LINE=
for /F "delims=" %%I IN ('@cscript.exe //NoLogo "%PROPS_FILE%_aux3.vbs" "%INF_LOG_DIR%\%REMEMBERED_FILE_NAME%"') DO (set FILE_FIRST_LINE=%%I)

echo MAX_TIMESTAMP=%MAX_TIMESTAMP%>"%PROPS_FILE%"
echo _domainname=%USERDOMAIN%>>"%PROPS_FILE%"
echo FILE_FIRST_LINE_%REMEMBERED_FILE_NAME%="%FILE_FIRST_LINE%">>"%PROPS_FILE%"
echo FILE_SIZE_%REMEMBERED_FILE_NAME%=%REMEMBERED_FILE_SIZE%>>"%PROPS_FILE%"
goto :do_non_initial_collect
goto :EOF


:is_it_remembered_log_file
set FILE_FIRST_LINE=
for /F "delims=" %%I IN ('@cscript.exe //NoLogo "%PROPS_FILE%_aux3.vbs" "%INF_LOG_DIR%\%1"') DO (set FILE_FIRST_LINE=%%I)
echo "%FILE_FIRST_LINE%" | find %2>nul
if NOT errorlevel 1 ( exit /b 1 )
exit /b 0
goto :EOF


rem ############################################################################
rem      Auxiliary VBS code for retrieving log files, which have the file modification timestamp equal to or greater than the given timestamp.
rem      arg(0) - The Informix Log Directory;
rem      arg(1) - Name of file with list of to-be-processed files
rem      arg(2) - The timestamp (e.g. "20100311154526") for log files selection.
rem ############################################################################
Set arg = WScript.Arguments                            'VBS_INFORMIX
Set fso = CreateObject("Scripting.FileSystemObject")   'VBS_INFORMIX
Set infile = fso.OpenTextFile(arg(1), 1)               'VBS_INFORMIX
Do While infile.AtEndOfStream <> True                  'VBS_INFORMIX
  CurFileName = arg(0) & "\" & infile.ReadLine         'VBS_INFORMIX
  Set f = fso.GetFile(CurFileName)                     'VBS_INFORMIX
  yyyy = DatePart("yyyy",f.DateLastModified)           'VBS_INFORMIX
  mm = Right(0 & DatePart("m",f.DateLastModified), 2)  'VBS_INFORMIX
  dd = Right(0 & DatePart("d",f.DateLastModified), 2)  'VBS_INFORMIX
  hh = Right(0 & DatePart("h",f.DateLastModified), 2)  'VBS_INFORMIX
  mn = Right(0 & DatePart("n",f.DateLastModified), 2)  'VBS_INFORMIX
  ss = Right(0 & DatePart("s",f.DateLastModified), 2)  'VBS_INFORMIX
  datetimeStamp = yyyy & mm & dd & hh & mn & ss        'VBS_INFORMIX
  If datetimeStamp >= arg(2) Then                      'VBS_INFORMIX
    WScript.Echo f.Name & ";" & datetimeStamp          'VBS_INFORMIX
  End If                                               'VBS_INFORMIX
Loop                                                   'VBS_INFORMIX
infile.Close                                           'VBS_INFORMIX


rem ############################################################################
rem      Auxiliary VBS code for retrieving the file modification timestamp (e.g. "20100311154526") of the given log file
rem ############################################################################
Set arg = WScript.Arguments                          'VBS_2_INFORMIX
Set fso = CreateObject("Scripting.FileSystemObject") 'VBS_2_INFORMIX
Set f = fso.GetFile(arg(0))                          'VBS_2_INFORMIX
yyyy = DatePart("yyyy",f.DateLastModified)           'VBS_2_INFORMIX
mm = Right(0 & DatePart("m",f.DateLastModified), 2)  'VBS_2_INFORMIX
dd = Right(0 & DatePart("d",f.DateLastModified), 2)  'VBS_2_INFORMIX
hh = Right(0 & DatePart("h",f.DateLastModified), 2)  'VBS_2_INFORMIX
mn = Right(0 & DatePart("n",f.DateLastModified), 2)  'VBS_2_INFORMIX
ss = Right(0 & DatePart("s",f.DateLastModified), 2)  'VBS_2_INFORMIX
datetimeStamp = yyyy & mm & dd & hh & mn & ss        'VBS_2_INFORMIX
WScript.Echo datetimeStamp                           'VBS_2_INFORMIX


rem ############################################################################
rem      Auxiliary VBS code for retrieving first line from the given log file.
rem ############################################################################
Set arg = WScript.Arguments                            'VBS_3_INFORMIX
Set fso = CreateObject("Scripting.FileSystemObject")   'VBS_3_INFORMIX
Set infile = fso.OpenTextFile(arg(0), 1)               'VBS_3_INFORMIX
If infile.AtEndOfStream <> True Then                   'VBS_3_INFORMIX
  WScript.Echo infile.ReadLine                         'VBS_3_INFORMIX
End If                                                 'VBS_3_INFORMIX 
infile.Close                                           'VBS_3_INFORMIX
