#!/bin/ksh
# Licensed Materials - Property of IBM
# 5724-S67
# (c) Copyright IBM Corp. 1998, 2010   All Rights Reserved.
# US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
export LC_TIME="C"

cea_exit()
{
  debug "$1 `date`"
  echo "CeA/ExtendIT Debug Log finished by $SCRIPTNAME on `hostname` (`uname`) after FAILED collect, `date`." >> ${CEA_GENACT_DEBUG_LOG}
  rm -f ${TEMPFILE1} 2>>$CEA_GENACT_DEBUG_LOG
  exit $2
}

#
# Function: mton MONTH_STRING_VALUE
# Input: month as string
# Return: echoed month number
# Copied from getnewrecs.sh
mton () 
{
  result="00"
  case "$1" in
    'Jan')
       result="01"
       ;;
    'Feb')
       result="02"
       ;;
    'Mar')
       result="03"
       ;;
    'Apr')
       result="04"
       ;;
    'May')
       result="05"
       ;;
    'Jun')
       result="06"
       ;;
    'Jul')
       result="07"
       ;;
    'Aug')
       result="08"
       ;;
    'Sep')
       result="09"
       ;;
    'Oct')
       result="10"
       ;;
    'Nov')
       result="11"
       ;;
    'Dec')
       result="12"
       ;;
     *)
       debug "mton: unable to evaluate $1"
       ;;
  esac
  echo "$result"
}

#
# Function: dton DATE_OF_FILE
# Input: modtime of file as "ls -got" returned
#        <Month> <Day> <Hour>:<Min> 
#    or  <Month> <Day> <Year> in case file is more then one year old
#    or  <Year>-<Month>-<Day> <Hour>:<Min> in the new format
# Return: echoed transformed timestamp in format YYYYMMDDhhmm
# Copied from getnewrecs.sh
dton ()
{
  now=`date +%m%d%H%M`

  NewFormat=`echo $1 | grep '[0-9]*-[0-9]*-[0-9]* [0-9]*:[0-9]*'`
  if [ -n "$NewFormat" ]
  then
    result=`echo $1 | sed -e 's/[-: ]//g'`
  else
    Year=""
    Hour=`echo $1 | grep [0-2][0-9]:[0-5][0-9] | sed -e 's/.\{1,\}\([0-2][0-9]\):[0-5][0-9]/\1/'`
    if [ -z "$Hour" ]
    then
      Hour="00"
      Year=`echo $1 | grep [1-2][0-9][0-9][0-9] | sed -e 's/.\{1,\}\([1-2][0-9][0-9][0-9]\)/\1/'`
    fi

    Min=`echo $1 | grep [0-2][0-9]:[0-5][0-9] | sed -e 's/.\{1,\}[0-2][0-9]:\([0-5][0-9]\)/\1/'`
    test -z "$Min" && Min="00"

    Month=`echo $1 | sed -e 's/.\{0,\}\([A-Z][a-z][a-z]\).\{1,\}/\1/'`
    Month=`mton $Month`

    Day=`echo $1 | grep " [0-3][0-9] " | sed -e 's/[A-Z][a-z][a-z] \{1,\}\([0-3][0-9]\).\{1,\}/\1/'`
    test -z "$Day" && Day=`echo $1 | sed -e 's/[A-Z][a-z][a-z] \{1,\}\([1-9]\).\{1,\}/0\1/'`

    if [ -z "$Year" ]
    then
      Year=`date +%Y`
      test "$now" -lt "$Month""01""$Hour""$Min" && Year=`expr $Year - 1`
    fi
  
    result="$Year""$Month""$Day""$Hour""$Min"
  fi

  echo "$result"
}

#
# Function: isLesser YYYYMMDDhhmm YYYYMMDDhhmm
# Input: two timestamps to be compared in format YYYYMMDDhhmm
# Return: 0 - $1 lesser than $2; 1 - $1 is not lesser then $2 
# Note: this function is used instead of `test $1 -lt $2` 
# because on HP-UX `test` works wrong for large numbers
# Copied from getnewrecs.sh
isLesser ()
{
  beg=1
  end=4
  while [ "$beg" -lt "10" ]
  do
    lval=`echo $1 | cut -c $beg-$end`
    rval=`echo $2 | cut -c $beg-$end`
    test "$lval" -lt "$rval" && return 0
    test "$rval" -lt "$lval" && return 1
    beg=`expr $beg + 4`
    end=`expr $end + 4`
  done
  return 1  
}

#
# Function: collect_actual_log_files
# Purpose: generic function that performs actual data collection.
# Input: none, ${ACTUAL_FILES} variable should specify to be processed log files.
# Return: none
collect_actual_log_files()
{
  if [ -z "${ACTUAL_FILES}" ] 
  then
    debug "Nothing to collect."
    touch "${OUTPUTFILE}"
    if [ ! -r "${PROPSFILE}" ]
    then
      MAX_TIMESTAMP=`ls -got "${PROPSFILE}".tmp | grep "^-" | ${SA_AWK} '{ print $4"-"$5"-"$6 }'`
      debug "Remembering the current timestamp (${MAX_TIMESTAMP}) for the next collect."
      echo MAX_TIMESTAMP="${MAX_TIMESTAMP}">>"${PROPSFILE}"
      rm -f "${PROPSFILE}".tmp
    fi
  else
    FILES_INFO=`ls -got ${ACTUAL_FILES} | grep "^-" | ${SA_AWK} '{ print $7"###"$4"-"$5"-"$6 }'`
    
    MAX_TIMESTAMP=`echo "${FILES_INFO}" | head -1 | sed -e "s/^.*###\(.*\)$/\1/"`
    debug "The most recent file modification timestamp: ${MAX_TIMESTAMP}."
    
    FILES_TO_COLLECT=""
    for aa in `echo "${FILES_INFO}" | sed -e "s/^\(.*\)###.*$/\1/"`
    do
      FILES_TO_COLLECT="${aa} ${FILES_TO_COLLECT}"
    done

    rm -f "${OUTPUTFILE}"
    rm -f "${PROPSFILE}".tmp
    for aa in ${FILES_TO_COLLECT}
    do
      HANDLE_LIVE_TRAIL="no"
      FILE_TIMESTAMP=`echo "${FILES_INFO}" | grep "${aa}###" | sed -e "s/^.*###\(.*\)$/\1/"`
      
      REMEMBERED_FIRST_LINE=""
      [[ -r "${PROPSFILE}" ]] && REMEMBERED_FIRST_LINE=`cat "${PROPSFILE}" | grep "^FILE_FIRST_LINE_${aa}=" | sed -e "s/^[^=]*=\(.*\)/\1/"`

      if [ -z "${REMEMBERED_FIRST_LINE}" ]
      then
        debug "  Collecting ${aa}, file modification timestamp: ${FILE_TIMESTAMP}."
	HANDLE_LIVE_TRAIL="yes"
        CURRENT_FIRST_LINE_LIVE=`head -n 1 "${aa}"`
	CURRENT_FILE_SIZE=`ls -l "${aa}" | ${SA_AWK} '{print $5}'`
	#head -c "${CURRENT_FILE_SIZE}" "${aa}" >>"${OUTPUTFILE}" 2>>${CEA_GENACT_DEBUG_LOG}
    dd if="${aa}" bs=${CURRENT_FILE_SIZE} count=1 >>"${OUTPUTFILE}" 2>>${CEA_GENACT_DEBUG_LOG}
#        cat "${aa}">>"${OUTPUTFILE}" 2>>${CEA_GENACT_DEBUG_LOG}
        [[ $? -ne 0 ]] && cea_exit "Unable to perform head on ${aa}. Collect is failed." "7"
      else
        CURRENT_FIRST_LINE=`head -n 1 "${aa}"`
        if [ "${REMEMBERED_FIRST_LINE}" != "${CURRENT_FIRST_LINE}" ]
        then
          debug "  Collecting ${aa}, file modification timestamp: ${FILE_TIMESTAMP}."
	  HANDLE_LIVE_TRAIL="yes"
          CURRENT_FIRST_LINE_LIVE=`head -n 1 "${aa}"`
	  CURRENT_FILE_SIZE=`ls -l "${aa}" | ${SA_AWK} '{print $5}'`
          #head -c "${CURRENT_FILE_SIZE}" "${aa}" >>"${OUTPUTFILE}" 2>>${CEA_GENACT_DEBUG_LOG}
          dd if="${aa}" bs=${CURRENT_FILE_SIZE} count=1 >>"${OUTPUTFILE}" 2>>${CEA_GENACT_DEBUG_LOG}
#          cat "${aa}">>"${OUTPUTFILE}" 2>>${CEA_GENACT_DEBUG_LOG}
          [[ $? -ne 0 ]] && cea_exit "Unable to perform head on ${aa}. Collect is failed." "8"  
        else
	  HANDLE_LIVE_TRAIL="yes"
          REMEMBERED_FILE_SIZE=`cat "${PROPSFILE}" | grep "^FILE_SIZE_${aa}=" | sed -e "s/^[^=]*=\(.*\)/\1/"`
          CURRENT_FIRST_LINE_LIVE=`head -n 1 "${aa}"`
          CURRENT_FILE_SIZE=`ls -l "${aa}" | ${SA_AWK} '{print $5}'`
          BYTES_TO_COLLECT=`expr "${CURRENT_FILE_SIZE}" "-" "${REMEMBERED_FILE_SIZE}"`
          debug "  Collecting ${aa} skipping first ${REMEMBERED_FILE_SIZE} bytes, file modification timestamp: ${FILE_TIMESTAMP}."
#          ${SA_TAIL} -c "${BYTES_TO_COLLECT}" "${aa}">>"${OUTPUTFILE}" 2>>${CEA_GENACT_DEBUG_LOG}
#          ${SA_TAIL} -c "+${REMEMBERED_FILE_SIZE}" "${aa}" | head -c "${BYTES_TO_COLLECT}" >>"${OUTPUTFILE}" 2>>${CEA_GENACT_DEBUG_LOG} 
          touch "${PROPSFILE}".container
          dd if="${aa}" bs=${CURRENT_FILE_SIZE} count=1 2>>${CEA_GENACT_DEBUG_LOG} >"${PROPSFILE}".container
          [ $? -ne 0 ]] && cea_exit "Unable to perform dd on ${aa}. Collect is failed." "9" 
          dd if="${PROPSFILE}.container" bs=${REMEMBERED_FILE_SIZE} skip=1 >>"${OUTPUTFILE}" 2>>${CEA_GENACT_DEBUG_LOG}
          [ $? -ne 0 ]] && cea_exit "Unable to perform dd on ${aa}. Collect is failed." "9" 
          rm "${PROPSFILE}".container 
        fi
      fi
      
      # Need to remember the following information for the next collect:
      #   - the greatest file modification timestamp;
      #   - name of every non-empty collected file, which have the file modification time equal to the greatest file modification timestamp;
      #   - first record of every non-empty remembered file;
      #   - size of every non-empty remembered file.
      if ([ "${FILE_TIMESTAMP}" = "${MAX_TIMESTAMP}" ] && [ -s "${aa}" ])
      then
	if [ "${HANDLE_LIVE_TRAIL}" = "yes" ]
	then
          FILE_FIRST_LINE="${CURRENT_FIRST_LINE_LIVE}"
	  FILE_SIZE="${CURRENT_FILE_SIZE}"
	else  
          FILE_FIRST_LINE=`head -1 "${aa}"`
          FILE_SIZE=`ls -l "${aa}" | ${SA_AWK} '{print $5}'`
	fi
        echo FILE_FIRST_LINE_"${aa}"="${FILE_FIRST_LINE}">>"${PROPSFILE}".tmp
        echo FILE_SIZE_"${aa}"="${FILE_SIZE}">>"${PROPSFILE}".tmp
      fi
    done
    
    cat "${PROPSFILE}".tmp > "${PROPSFILE}"
    rm -f "${PROPSFILE}".tmp
    echo MAX_TIMESTAMP="${MAX_TIMESTAMP}">>"${PROPSFILE}"
    [[ -s "${OUTPUTFILE}" ]] && touch "${OUTPUTFILE}"
  fi
}

#
# Function: do_initial_collection
# Purpose: performs initial collect, collects relevant files ("${INSTNAME}.[0-9][0-9]*$") from the specified directory (${LOGDIR})  
#               which were modified during the last elapsed 2 days.
# Input: none
# Return: none
do_initial_collection()
{
  touch "${PROPSFILE}".tmp
  debug "Going to perform initial collect (${PROPSFILE} does not exist)."
  ACTUAL_FILES=`find "${LOGDIR}" -name "${INSTNAME}.*" -mtime -2 | grep "${LOGDIR}/${INSTNAME}.[0-9][0-9]*$"`
  collect_actual_log_files
}

#
# Function: do_non_initial_collection_new
# Purpose: performs non-initial collect using state information produced by the current version of this script.
# Input: none
# Return: none
do_non_initial_collection_new()
{
  ACTUAL_FILES=""
    
  # Converts "Feb-16-15:28" to "Feb 16 15:28"
  PREV_MAX_TIMESTAMP_OK=`echo "${PREV_MAX_TIMESTAMP}" | sed -e 's/-/ /g'`

  # Converts "Feb 16 15:28" to "201002161528"
  PREV_MAX_TIMESTAMP_OK=`dton "${PREV_MAX_TIMESTAMP_OK}"`
    
  FOUND_FILES=`find "${LOGDIR}" -name "${INSTNAME}.*" | grep "${LOGDIR}/${INSTNAME}.[0-9][0-9]*$"`
  if [ ! -z "${FOUND_FILES}" ] 
  then
    FOUND_FILES_INFO=`ls -got ${FOUND_FILES} | grep "^-" | ${SA_AWK} '{ print $7"###"$4"-"$5"-"$6 }'`
    for aa in ${FOUND_FILES_INFO}
    do
      # Converts "/tmp/informix/demo_on.8###Feb-16-15:28" to "Feb 16 15:28"
      FOUND_FILE_TIMESTAMP=`echo "${aa}" | sed -e "s/^.*###\(.*\)$/\1/" | sed -e 's/-/ /g'`     
      # Converts "Feb 16 15:28" to "201002161528"
      FOUND_FILE_TIMESTAMP_OK=`dton "${FOUND_FILE_TIMESTAMP}"`

      isLesser "${FOUND_FILE_TIMESTAMP_OK}" "${PREV_MAX_TIMESTAMP_OK}" && break

      # Converts "/tmp/informix/demo_on.8###Feb-16-15:28" to "/tmp/informix/demo_on.8"
      FOUND_FILE_NAME=`echo "${aa}" | sed -e "s/^\(.*\)###.*$/\1/"`
      ACTUAL_FILES="${FOUND_FILE_NAME} ${ACTUAL_FILES}"
    done
  fi
  collect_actual_log_files
}

#
# Function: do_non_initial_collection_old
# Purpose: performs non-initial collect using state information produced by the previous version of this script.
# Input: none
# Return: none
do_non_initial_collection_old()
{
  # the previous version of this script stores 1-line state info, like: "demo_on.39:5:/tmp/informix"
  REMEMBERED_FILE_NAME=`cat ${PROPSFILE} | grep "^${INSTNAME}.[0-9][0-9]*" | sed -e "s/^\(${INSTNAME}\.[0-9][0-9]*\):[0-9][0-9]*:.*$/\1/"`
  
  if ( [ -z "${REMEMBERED_FILE_NAME}" ] || [ ! -r "${LOGDIR}/${REMEMBERED_FILE_NAME}" ] )
  then
    do_all_files_collection
  else
    CURRENT_FILE_LINES=`wc -l "${LOGDIR}"/"${REMEMBERED_FILE_NAME}" | sed -e 's/^ *\([0-9]\{1,\}\).*/\1/'`
    REMEMBERED_LINE_NUMBER=`cat ${PROPSFILE} | grep "^${INSTNAME}.[0-9][0-9]*" | sed -e "s/^${INSTNAME}\.[0-9][0-9]*:\([0-9][0-9]*\):.*$/\1/"`
    
    if ( [ -z "$REMEMBERED_LINE_NUMBER" ] || [ "$REMEMBERED_LINE_NUMBER" -gt "$CURRENT_FILE_LINES" ] )
    then
      do_all_files_collection
    else
      head -n "$REMEMBERED_LINE_NUMBER" "${LOGDIR}/${REMEMBERED_FILE_NAME}" > "${PROPSFILE}".tmp
      FILE_SIZE=`ls -l "${PROPSFILE}".tmp | ${SA_AWK} '{print $5}'`
      rm -f "${PROPSFILE}".tmp
      
      PREV_MAX_TIMESTAMP=`ls -got "${LOGDIR}/${REMEMBERED_FILE_NAME}" | grep "^-" | ${SA_AWK} '{ print $4"-"$5"-"$6 }'`
      FILE_FIRST_LINE=`head -1 "${LOGDIR}/${REMEMBERED_FILE_NAME}"`
      
      echo MAX_TIMESTAMP="${PREV_MAX_TIMESTAMP}">"${PROPSFILE}"
      echo FILE_FIRST_LINE_"${LOGDIR}/${REMEMBERED_FILE_NAME}"="${FILE_FIRST_LINE}">>"${PROPSFILE}"
      echo FILE_SIZE_"${LOGDIR}/${REMEMBERED_FILE_NAME}"="${FILE_SIZE}">>"${PROPSFILE}"
      
      do_non_initial_collection_new
      fi
  fi
}

#
# Function: do_all_files_collection
# Purpose: collects all relevant files ("${INSTNAME}.[0-9][0-9]*$") from the specified directory (${LOGDIR}).
# Input: none
# Return: none
do_all_files_collection()
{
  debug "Going to collect all relevant log files from the specified directory (${LOGDIR}). Previously stored state information is not applicable."
  ACTUAL_FILES=`find "${LOGDIR}" -name "${INSTNAME}.*" | grep "${LOGDIR}/${INSTNAME}.[0-9][0-9]*$"`
  collect_actual_log_files
}

#
# MAIN BODY
#
if [ -z "${PROPSFILE}" ]
then
  echo "The event source property file (\$PROPSFILE) is not specified. Collect is failed."
  exit 1
fi

CEA_GENACT_DEBUG_LOG=../log/${PROPSFILE}.log

. ${CEA_GENACT_FUNCLIB} >${CEA_GENACT_DEBUG_LOG} 2>>${CEA_GENACT_DEBUG_LOG}
[[ $? -ne 0 ]] && cea_exit "funclib.sh is not found" "2"

OUTPUTFILE="${TEMPFILE1}"
LOGDIR="${OPTION2}"
INSTNAME="${OPTION3}"
LOGREMOVE="${OPTION4}"

debug_init

[[ -z "${OUTPUTFILE}" ]] && cea_exit "The name of output file is not specified. Collect is failed." "3"
[[ -z "${LOGDIR}" ]] && cea_exit "The Informix Log Directory is not specified. Collect is failed." "4"
[[ -z "${INSTNAME}" ]] && cea_exit "The Informix Server Name is not specified. Collect is failed." "5"
[[ -r "${LOGDIR}" ]] || cea_exit "Do not have read access on ${LOGDIR}. Collect is failed." "6"

SA_TAIL="tail"
SA_AWK="awk"
if [ "`uname`" = "SunOS" ]; then
  SA_TAIL="/usr/xpg4/bin/tail"
  SA_AWK="/usr/xpg4/bin/awk"
fi

# LOGS COLLECTION
if [ ! -r "${PROPSFILE}" ]
then
  do_initial_collection
else
  PREV_MAX_TIMESTAMP=`cat "${PROPSFILE}" | grep "^MAX_TIMESTAMP=" | sed -e "s/^MAX_TIMESTAMP=\(.*\)/\1/"`
  if [ -z "${PREV_MAX_TIMESTAMP}" ]
  then
    debug "Going to perform non initial collect using state information produced by the previous version of this script."
    do_non_initial_collection_old
  else
    debug "Going to perform non initial collect starting from ${PREV_MAX_TIMESTAMP}."
    do_non_initial_collection_new
  fi
fi

# LOGS DELETING
if [ ! -z "${LOGREMOVE}" ]
then
  DAYS_TO_REMOVE=`echo "${LOGREMOVE}" | grep "^[0-9][0-9]*$"`
  if [ ! -z "${DAYS_TO_REMOVE}" ]
  then
    if [ "${DAYS_TO_REMOVE}" -eq "0" ]
    then
      debug "Going to delete all log files produced by the specified Informix Server (${INSTNAME}) from the specified directory (${LOGDIR}), except active log files."
      FILES_TO_REMOVE=`find "${LOGDIR}" -name "${INSTNAME}.*" | grep "${LOGDIR}/${INSTNAME}.[0-9][0-9]*$"`
    else
      debug "Going to delete log files produced by the specified Informix Server (${INSTNAME}) from the specified directory (${LOGDIR}), which were not modified during elapsed ${DAYS_TO_REMOVE} days."
      FILES_TO_REMOVE=`find "${LOGDIR}" -name "${INSTNAME}.*" ! -mtime -"${DAYS_TO_REMOVE}" | grep "${LOGDIR}/${INSTNAME}.[0-9][0-9]*$"`
    fi

    if [ -z "${FILES_TO_REMOVE}" ] 
    then
      debug "  Nothing to delete."
    else
      for aa in ${FILES_TO_REMOVE}
      do
        IS_ACTIVE_FILE=`cat "${PROPSFILE}" | grep "^FILE_SIZE_${aa}="`
        if [ ! -z "${IS_ACTIVE_FILE}" ]
        then
          debug "  Skipping \"${aa}\" from deleting. It is active log file."
        else
          debug "  Deleting \"${aa}\"."
          rm -f "${aa}" 2>>${CEA_GENACT_DEBUG_LOG}
        fi
      done
    fi
  fi
fi

echo "CeA/ExtendIT Debug Log finished by $SCRIPTNAME on `hostname` (`uname`) after SUCCESSFUL collect, `date`." >> ${CEA_GENACT_DEBUG_LOG}
exit 0
