#!/bin/ksh
TEMP_FILE=$1
HOST=""
PORT=$3
USER_NAME=$4
PASSWORD=$5
TIP_INSTALL_DIR=$6
PROPS_FILE=$7

touch $TEMP_FILE

if [ -z $PROPS_FILE]; then
   act_log_file="../log/gettklmgroups.err"
   echo "[`date`] Failure: Missing property file argument">>$act_log_file
   echo "[`date`] Processing finished. [FAILURE]">>$act_log_file
   exit 1
fi
[ -f "$PROPS_FILE" ] || touch "$PROPS_FILE"
act_log_file="../log/`basename $PROPS_FILE`.log"
echo "[`date`] Processing start. ">$act_log_file
HOST=`grep "SystemName" $PROPS_FILE | awk '{print $3}'`
if [ -z $HOST]; then
   echo "[`date`] Failure: Missing parameter SystemName from PROPS_FILE">>$act_log_file
   echo "[`date`] Processing finished (Return code 1)">>$act_log_file
   exit 1
fi

if [ -z $PORT]; then
   echo "[`date`] Failure: Missing WAS repository SOAP port argument">>$act_log_file
   echo "[`date`] Processing finished. [FAILURE]">>$act_log_file
   exit 1
fi

if [ -z $USER_NAME]; then
   echo "[`date`] Warning: Missing Repository User argument">>$act_log_file
fi

if [ -z $PASSWORD]; then
   echo "[`date`] Warning: Missing Repository Password argument">>$act_log_file
fi

if [ -z $TIP_INSTALL_DIR]; then
   echo "[`date`] Failure: Missing WAS installation directory argument">>$act_log_file
   echo "[`date`] Processing finished. [FAILURE]">>$act_log_file
   exit 1
fi                      

if [ -f "$TIP_INSTALL_DIR/properties/fsdb/_was_profile_default/default.sh" ]; then
    echo "[`date`] WebSphere component ($TIP_INSTALL_DIR/properties/fsdb/_was_profile_default/default.sh) found.">>$act_log_file
    . $TIP_INSTALL_DIR/properties/fsdb/_was_profile_default/default.sh>>$act_log_file
else
   echo "[`date`] WebSphere component  ('WAS installation directory'/properties/fsdb/_was_profile_default/default.sh) not found. Check 'WAS installation directory' ES property. [FAILURE]">>$act_log_file
   exit 1
fi

if [ -f "$WAS_USER_SCRIPT" ]; then
    echo "[`date`] WebSphere component <$WAS_USER_SCRIPT> found.">>$act_log_file
    . $WAS_USER_SCRIPT>>$act_log_file; 
else
   echo "[`date`] WebSphere component <$WAS_USER_SCRIPT> not found. [FAILURE]">>$act_log_file
   exit 1
fi

JAVA_BIN="$JAVA_HOME/bin/java"
if [ -f "$JAVA_BIN" ]; then
   echo "[`date`] Use TIP Java machine: "$JAVA_BIN"."  >>$act_log_file
else
   echo "[`date`] Can not find TIP Java machine: "$JAVA_BIN"."  >>$act_log_file
   exit 1
fi
TRUSTFILE="$WAS_HOME/profiles/TIPProfile/etc/DummyClientTrustFile.jks"
KEYFILE="$WAS_HOME/profiles/TIPProfile/etc/DummyClientKeyFile.jks"

if [ -f "$TRUSTFILE" ]; then
    echo "[`date`] Trustfile <$TRUSTFILE> found.">>$act_log_file
else
   echo "[`date`] Warning: Trustfile <$TRUSTFILE> was not found. ">>$act_log_file
   TRUSTFILE="remote"
fi
if [ -f "$KEYFILE" ]; then
    echo "[`date`] Keyfile <$KEYFILE> found.">>$act_log_file
else
   echo "[`date`] Warning: Keyfile <$KEYFILE> was not found.">>$act_log_file
   KEYFILE="remote"
fi

if [ -z $CLIENTSSL]; 
then
# V6.0
"$JAVA_BIN" \
   -Djava.security.auth.login.config="$WAS_HOME/properties/wsjaas_client.conf" \
   -Djava.ext.dirs="$JAVA_HOME/jre/lib/ext:$WAS_HOME/plugins:$WAS_EXT_DIRS" \
   -Dserver.root="$WAS_HOME" \
   -Duser.install.root="$USER_INSTALL_ROOT" \
   -classpath "$WAS_CLASSPATH:$WAS_HOME/lib/admin.jar:$WAS_HOME/lib/utils.jar:$WAS_HOME/lib/bootstrap.jar:$WAS_HOME/lib/j2ee.jar:../bin/gettklmgroups.jar:." \
   nl.consul.cea.gensub.itklmuis.java.GetTKLMGroups \
   -host $HOST \
   -port $PORT \
   -username $USER_NAME \
   -password $PASSWORD \
   -trustfile $TRUSTFILE \
   -keyfile $KEYFILE \
   -chunk $TEMP_FILE \
   -propsfile $PROPS_FILE >> $act_log_file 2>&1
code=$?
   if [ $code = 0 ]; then
     echo "[`date`] Processing finished. [SUCCESS]" >> $act_log_file
   else
     echo "[`date`] Processing finished. [FAILURE]" >> $act_log_file
   fi

else
# V6.1 and 7.0

"$JAVA_BIN" \
   -Djava.security.auth.login.config="$WAS_HOME/properties/wsjaas_client.conf" \
   -Djava.ext.dirs="$JAVA_HOME/jre/lib/ext:$WAS_HOME/plugins:$WAS_EXT_DIRS" \
   -Dserver.root="$WAS_HOME" "$CLIENTSAS" "$CLIENTSSL" \
   -Duser.install.root="$USER_INSTALL_ROOT" \
   -classpath "$WAS_CLASSPATH:$WAS_HOME/lib/admin.jar:$WAS_HOME/lib/utils.jar:$WAS_HOME/lib/bootstrap.jar:$WAS_HOME/lib/j2ee.jar:../bin/gettklmgroups.jar:." \
   nl.consul.cea.gensub.itklmuis.java.GetTKLMGroups \
   -host $HOST \
   -port $PORT \
   -username $USER_NAME \
   -password $PASSWORD \
   -trustfile $TRUSTFILE \
   -keyfile $KEYFILE \
   -chunk $TEMP_FILE \
   -propsfile $PROPS_FILE >> $act_log_file 2>&1
code=$?
   if [ $code = 0 ]; then
     echo "[`date`] Processing finished. [SUCCESS]" >> $act_log_file
   else
     echo "[`date`] Processing finished. [FAILURE]" >> $act_log_file
   fi
fi