@echo off

setlocal
set TKLM_LOGSET=%~1
set LOGDIR=%~2
set LOGMASK=%~3
set USER_NAME=%~4
set USER_PWD=%~5
for %%I in (%*) do set PROPS_FILE=%%~I
for %%I in (%PROPS_FILE%) do set PROPS_NAME=%%~nI
set PLATFORM=Windows

set Path=%TSIEM_HOME%\jre\jre\bin;%Path%
rem -- sets actuator log name
if "%PROPS_FILE%" == "" (
   set ACTUATOR_LOG="..\log\gettklmlogs.err"
) else (
   set ACTUATOR_LOG="..\log\%PROPS_NAME%.log"
)

type nul > %ACTUATOR_LOG%

set PRINTING=enabled

set LOGSETNAME_EMPTY=0
if "%TKLM_LOGSET%" == "" (
   call :print_message "ERROR: Missing output file name argument for TKLM audit logs." %ACTUATOR_LOG%
   set LOGSETNAME_EMPTY=1
) else (
   type nul > "%TKLM_LOGSET%"
)
if %LOGSETNAME_EMPTY% EQU 1 goto failure

if "%PROPS_FILE%" == "" (
   call :print_message "ERROR: Missing property file argument." %ACTUATOR_LOG%
   goto failure
)
if not exist "%PROPS_FILE%" (
   call :print_message "ERROR: Can't find property file '%PROPS_FILE%'." %ACTUATOR_LOG%
   goto failure
)
call :print_message "Processing started." %ACTUATOR_LOG%

set TKLM_COLLECT=1
if "%LOGDIR%" == "" (
   call :print_message "WARNING: Missing log directory path name argument for TKLM audit log." %ACTUATOR_LOG%
   set TKLM_COLLECT=0
)
if "%LOGMASK%" == "" (
   call :print_message "WARNING: Missing template of the log file name argument for TKLM audit log." %ACTUATOR_LOG%
   set TKLM_COLLECT=0
)
if %TKLM_COLLECT% EQU 0 (
   call :print_message "ERROR: TKLM audit log cannot be processed due to missing required parameter." %ACTUATOR_LOG%
   goto failure
)

call :parsepropsfile
if "%HOST_NAME%" == "" (
   call :print_message "ERROR: 'SystemName' variable is not found in '%PROPS_FILE%' file." %ACTUATOR_LOG%
   goto :failure
)


set TSIEM_BIN_DIR=%~dp0%
set TSIEM_BIN_DIR=%TSIEM_BIN_DIR:~0,-1%
set PS_EXE=%TSIEM_BIN_DIR%\psexec.exe
set PS_OPTIONS=-u %USER_NAME% -p %USER_PWD% -q
set NET_OPTIONS=%USER_PWD% /u:%USER_NAME%
set LOCREMDEC_SCRIPT=%TSIEM_BIN_DIR%\localorremote.vbs
set JAR_FILE=gettklmlogs.jar
set TKLM_TMP_LOGSET=tklm_logset.txt

call :checkremotecollect || goto failure

if %REMOTE_COLLECT% EQU 1 (
   if "%USER_NAME%" == "" (
      call :print_message "ERROR: Missing username for accessing target system." %ACTUATOR_LOG%
      goto failure
   )
   if "%USER_PWD%" == "" (
      call :print_message "ERROR: Missing password for accessing target system." %ACTUATOR_LOG%
      goto failure
   )
)

call :print_message "- Start preparing environment for the collect process." %ACTUATOR_LOG%


if %REMOTE_COLLECT% EQU 0 goto localcollect

  
   call :print_message "- Start the remote collect process." %ACTUATOR_LOG%
   call :print_message "Find free drive letter for mapping network share." %ACTUATOR_LOG%
   call :findfreedisk || goto failure
   set COLLECT_DIR=%SHARE_DISK%\Temp\ES-%PROPS_NAME%

   call :print_message "Map network share of the target system on the drive '%SHARE_DISK%'." %ACTUATOR_LOG%
   net use %SHARE_DISK% /delete > nul
   net use %SHARE_DISK% \\%HOST_NAME%\ADMIN$ %NET_OPTIONS% > nul
   if not exist "%SHARE_DISK%\" (
      call :print_message "ERROR: Failure during mapping network share of the target system." %ACTUATOR_LOG%
      goto failure
   )

   if exist "%COLLECT_DIR%" rmdir /S /Q "%COLLECT_DIR%" > nul
   mkdir "%COLLECT_DIR%" > nul

   call :print_message "Copy necessary execution files to the target system." %ACTUATOR_LOG%
     
   rem -- Copying propsfile  to the target system
   copy /Y "%PROPS_FILE%" "%COLLECT_DIR%" > nul
   if errorlevel 1 (
       call :print_message "ERROR: Failure to copy '%PROPS_FILE%' to the target system." %ACTUATOR_LOG%
       goto failure
    ) else (
       call :print_message "File '%PROPS_FILE%' successfully copied to the target system." %ACTUATOR_LOG%
    )
   rem -- Copying necessary binary files to the target system
   call :copy2target "%TSIEM_BIN_DIR%" "%JAR_FILE%" || goto failure
   
   call :print_message "- Start the collect process." %ACTUATOR_LOG%
   rem -- Start collect script on target system and getting result files from target system
   SET TARGET_DIR=%%SystemRoot%%\Temp\ES-%PROPS_NAME%
   echo @java.exe -Dfile.encoding=UTF-8 -classpath ".;%TARGET_DIR%\%JAR_FILE%" nl.consul.cea.gensub.itklm.java.GetTKLMLogs "%LOGDIR%" "%LOGMASK%" "%TARGET_DIR%\%TKLM_TMP_LOGSET%" "%TARGET_DIR%\%PROPS_NAME%.props" "%PLATFORM%"> "%COLLECT_DIR%\collectscript.bat"
   "%PS_EXE%" \\%HOST_NAME% %PS_OPTIONS% cmd.exe /c "%TARGET_DIR%\collectscript.bat">> %ACTUATOR_LOG%
   
   rem -- Check for collect script errors on target system
   if exist "%COLLECT_DIR%\%PROPS_NAME%.props" (
   copy /B /Y "%COLLECT_DIR%\%PROPS_NAME%.props" "%PROPS_FILE%" > nul
   if errorlevel 1 (
         call :print_message "ERROR: Failure to get new propsfile from the target system." %ACTUATOR_LOG%
         goto failure
      )
   ) else (
      call :print_message "ERROR: There is not exist new propsfile on the target system." %ACTUATOR_LOG%
      goto failure
   )
   if exist "%COLLECT_DIR%\%TKLM_TMP_LOGSET%" (
      copy /B /Y "%COLLECT_DIR%\%TKLM_TMP_LOGSET%" "%TKLM_LOGSET%" > nul
      if errorlevel 1 (
         call :print_message "ERROR: Failure to copy TKLM collected logset from the target system." %ACTUATOR_LOG%
         goto failure
      )
   ) else (
      call :print_message "WARNING: There is not exist TKLM collected logset on target system. Possible reason: new events are not presented in logs." %ACTUATOR_LOG%
   )
       
   if exist "%COLLECT_DIR%" rmdir /S /Q "%COLLECT_DIR%" > nul
   net use %SHARE_DISK% /delete > nul
   goto finish
   
:localcollect
 
 call :print_message "- Start the local collect process." %ACTUATOR_LOG%
   rem -- Start collect script
    
   java -Dfile.encoding=UTF-8 -classpath ".;%TSIEM_BIN_DIR%\gettklmlogs.jar" nl.consul.cea.gensub.itklm.java.GetTKLMLogs "%LOGDIR%" "%LOGMASK%" "%TKLM_LOGSET%" "%PROPS_FILE%" %PLATFORM% 1>> %ACTUATOR_LOG% 2>&1
    if errorlevel 1 (
      call :print_message "ERROR: Collect process failed." %ACTUATOR_LOG%
      goto failure
  )
  
:finish

call :cleanup
call :print_message "Processing finished [SUCCESS]." %ACTUATOR_LOG%
exit /b 0

:failure

call :cleanup
call :print_message "Processing finished [FAILURE]." %ACTUATOR_LOG%
exit /b 1


:parsepropsfile
rem -- read value of the 'SystemName' parameter into HOST_NAME
for /f "tokens=2 delims==" %%I in ('findstr /R /C:"^SystemName = .*" "%PROPS_FILE%" 2^>nul') do set HOST_NAME=%%I
for /f %%I in ("%HOST_NAME%") do set HOST_NAME=%%I
exit /b

rem #########################################################################################
rem Check collecting mode and set REMOTE_COLLECT variable
rem #########################################################################################

:checkremotecollect

for /f "delims=" %%I in ('cscript.exe //Nologo "%LOCREMDEC_SCRIPT%" %HOST_NAME% 2^>nul') do set RESULT=%%I
for /f "tokens=1,2 delims=|" %%a in ("%RESULT%") do set RESULT1=%%a&set RESULT2=%%b

if "%RESULT1%" == "LOCAL" (
   set REMOTE_COLLECT=0
   call :print_message "Collect from the local target system: '%HOST_NAME%'." %ACTUATOR_LOG%
) else (
  if "%RESULT1%" == "REMOTE" (
     set REMOTE_COLLECT=1
     set HOST_NAME=%RESULT2%
     call :print_message "Collect from the remote target system: '%HOST_NAME%'." %ACTUATOR_LOG%
  ) else (
     call :print_message "ERROR: Script for determining mode of the collect return failure." %ACTUATOR_LOG%
     call :print_message "%RESULT2%" %ACTUATOR_LOG%
     exit /b 1
  )
)
exit /b

rem ######################################################################################
rem printing routine
rem prints a message to actuator log. 
rem takes two parameters: %1 - message itself, %2 - name of a log file
rem ######################################################################################

:print_message

if "%PRINTING%" == "enabled" @echo [%date%][%time%] %~1>> %2
exit /b

rem ######################################################################################
rem Searching for unused drive letter for mapping shared resource
rem ######################################################################################

:findfreedisk

for %%I in (Z Y X W V U T S R Q P O N M L K) do (
    if not exist "%%I:\" (
        cd %%I: > nul 2>&1
        if errorlevel 1 (
            set SHARE_DISK=%%I:
            exit /b 0
        )
    )
)

call :print_message "ERROR: Could not find unused drive letter for mapping target network share." %ACTUATOR_LOG%
exit /b 1


rem ######################################################################################
rem Copy content to logfile
rem ######################################################################################

:print2log

if exist "%~1" (
  for /f "delims=" %%I in ('type "%~1" 2^>nul') do (
    call :print_message "%HOST_NAME%: %%I" %ACTUATOR_LOG%
  )
)
exit /b


rem ######################################################################################
rem Copy files to the target system for execution
rem ######################################################################################

:copy2target

for %%I in (%~2) do (
    copy /Y "%~1\%%I" "%COLLECT_DIR%" > nul
    if errorlevel 1 (
       call :print_message "ERROR: Failure to copy '%%I' to the target system." %ACTUATOR_LOG%
       exit /b 1
    ) else (
       call :print_message "File '%%I' successfully copied to the target system." %ACTUATOR_LOG%
    )
)
exit /b 0


rem ######################################################################################
rem Cleanup routine. Deleting temp directories and unmaping share
rem ######################################################################################

:cleanup

call :print_message "Free and clean up all used system resources." %ACTUATOR_LOG%
if not "%COLLECT_DIR%" == "" (
   if exist "%COLLECT_DIR%" rmdir /S /Q "%COLLECT_DIR%" > nul
)
if not "%SHARE_DISK%" == "" (
   if exist "%SHARE_DISK%\" net use "%SHARE_DISK%" /delete > nul
)
exit /b
