#!/bin/ksh
TEMP_FILE=$1
# WAS parameters
#set WAS_HOST=%2
WAS_PORT=$3
WAS_USER_NAME=$4
WAS_PASSWORD=$5
WAS_INSTALL_DIR=$6


#LDAP parameters
LDAP_HOST=$7
LDAP_PORT=$8
LDAP_USER_OBJECTCLASS=$9
shift 1
LDAP_GROUP_OBJECTCLASS=$9
shift 1
LDAP_USER_BASEDN=$9
shift 1
LDAP_GROUP_BASEDN=$9
shift 1
LDAP_USERID_ATTRIBUTE=$9
shift 1
LDAP_AUTHID_ATTRIBUTE=$9
shift 1
LDAP_GROUPNAME_ATTRIBUTE=$9
shift 1
LDAP_SEARCH_DN=$9
shift 1
LDAP_SEARCH_PW=$9
shift 1
LDAP_GROUP_LOOKUP_METHOD=$9
shift 1
LDAP_GROUP_LOOKUP_ATTRIBUTE=$9
shift 1
DISABLE_LDAP=no

#DB2 parameters
DB_HOST=$9
shift 1
DB_PORT=$9
shift 1
DB_USERNAME=$9
shift 1
DB_PASSWORD=$9
shift 1
DB_NAME=$9
shift 1
DB_SCHEMA=$9
shift 1
DISABLE_DB=no

PROPS_FILE=$9

act_log_file="../log/`basename $PROPS_FILE`.log";

echo "[`date`] Processing start. ">$act_log_file;

if [ "$PROPS_FILE" = "" ]; 
then
   echo "[`date`] Failure: Missing property file argument">>$act_log_file;
   echo "[`date`] Processing finished. [FAILURE]">>$act_log_file;
   exit 1;
fi

WAS_HOST=`grep "SystemName" $PROPS_FILE | awk '{print $3}'`

if [ "$WAS_HOST" = "" ]; 
then
   echo "[`date`] Failure: Missing parameter SystemName from PROPS_FILE">>$act_log_file;
   echo "[`date`] Processing finished (Return code 1)">>$act_log_file;
   exit 1;
fi

if [ "$TEMP_FILE" = "" ]; 
then
   echo "[`date`] Failure: Missing output file argument">>$act_log_file;
   echo "[`date`] Processing finished. [FAILURE]">>$act_log_file;
   exit 1;
fi

if [ "$WAS_PORT" = "" ]; 
then
   echo "[`date`] Failure: Missing WAS repository SOAP port argument">>$act_log_file;
   echo "[`date`] Processing finished. [FAILURE]">>$act_log_file;
   exit 1;
fi

if [ "$WAS_USER_NAME" = "" ]; 
then
   echo "[`date`] Warning: Missing Repository User argument">>$act_log_file;
fi

if [ "$WAS_PASSWORD" = "" ]; 
then
   echo "[`date`] Warning: Missing Repository Password argument">>$act_log_file;
fi

if [ "$WAS_INSTALL_DIR" = "" ]; 
then
   echo "[`date`] Failure: Missing WAS installation directory argument">>$act_log_file;
   echo "[`date`] Processing finished. [FAILURE]">>$act_log_file;
   exit 1;
fi                      

if [ -f "$WAS_INSTALL_DIR/properties/fsdb/_was_profile_default/default.sh" ]; then
    echo "[`date`] WebSphere component ($WAS_INSTALL_DIR/properties/fsdb/_was_profile_default/default.sh) found.">>$act_log_file;
    . $WAS_INSTALL_DIR/properties/fsdb/_was_profile_default/default.sh>>$act_log_file;
else
   echo "[`date`] WebSphere component  ('WAS installation directory'/properties/fsdb/_was_profile_default/default.sh) not found. Check 'WAS installation directory' ES property. [FAILURE]">>$act_log_file;
   exit 1;
fi

if [ -f "$WAS_USER_SCRIPT" ]; then
    echo "[`date`] WebSphere component <$WAS_USER_SCRIPT> found.">>$act_log_file;
    . $WAS_USER_SCRIPT>>$act_log_file; 
else
   echo "[`date`] WebSphere component <$WAS_USER_SCRIPT> not found. [FAILURE]">>$act_log_file;
   exit 1;
fi

# V6.1
"$JAVA_HOME/bin/java"  -Djava.security.auth.login.config="$WAS_HOME/properties/wsjaas_client.conf" -Dws.ext.dirs="$WAS_EXT_DIRS;$WAS_USER_DIRS:." -Djava.naming.factory.initial=com.ibm.websphere.naming.WsnInitialContextFactory  -Dserver.root="$WAS_HOME" "$CLIENTSAS" "$CLIENTSSL" -Duser.install.root="$USER_INSTALL_ROOT" -classpath "../bin/gettspmgroups.jar:$WAS_CLASSPATH:$WAS_HOME/plugins/com.ibm.ws.security.crypto_6.1.0.jar:$WAS_HOME/bin/ProfileManagement/plugins/com.ibm.websphere.v61_6.1.0/ws_runtime.jar:$WAS_HOME/runtimes/com.ibm.ws.webservices.thinclient_6.1.0.jar:$WAS_HOME/plugins/com.ibm.ws.runtime_6.1.0.jar:$WAS_HOME/plugins/com.ibm.ws.admin.core.jar:$WAS_HOME/plugins/com.ibm.ws.prereq.soap.jar:$WAS_HOME/deploytool/itp/plugins/com.ibm.websphere.v61_6.1.200/ws_runtime.jar:$WAS_HOME/plugins/com.ibm.ws.security.crypto.jar:$WAS_HOME/runtimes/com.ibm.ws.webservices.thinclient_7.0.0.jar:$WAS_HOME/plugins/com.ibm.ws.admin.services.jar:$WAS_HOME/lib/bootstrap.jar:$WAS_HOME/plugins/com.ibm.ws.emf.jar:$WAS_HOME/plugins/org.eclipse.emf.ecore.jar:$WAS_HOME/plugins/org.eclipse.emf.common.jar:$WAS_HOME/plugins/com.ibm.ffdc.jar:$WAS_HOME/plugins/com.ibm.ws.prereq.javamail.jar:$WAS_HOME/runtimes/com.ibm.ws.ejb.thinclient_7.0.0.jar:." nl.consul.cea.gensub.itspmuis.java.GetTSPMGroups -host $WAS_HOST -port $WAS_PORT -username $WAS_USER_NAME -password $WAS_PASSWORD -chunk $TEMP_FILE -propsfile $PROPS_FILE -version 6.1 2>>$act_log_file 1>>$act_log_file
code=$?
if [ $code = 0 ];
then
   echo "[`date`] Collect from WAS is finished.">>$act_log_file;
else
   echo "[`date`] Processing finished. [FAILURE]">>$act_log_file;
   exit 1
fi

# collecting LDAP
if [ ! "$LDAP_HOST" = "" ]; 
then
     echo [`date`] Starting LDAP information collect. >> $act_log_file
     if [ "$LDAP_PORT" = "" ]; 
     then
         DISABLE_LDAP=yes
         echo [`date`] LDAP port is not defined. Skipping collect.>> $act_log_file
     fi
     if [ "$LDAP_USER_OBJECTCLASS" = "" ]; 
     then
         DISABLE_LDAP=yes
         echo [`date`] LDAP user objectclass is not defined. Skipping collect.>> $act_log_file
     fi
     if [ "$LDAP_GROUP_OBJECTCLASS" = "" ]; 
     then
         DISABLE_LDAP=yes
         echo [`date`] LDAP group objectclass is not defined. Skipping collect.>> $act_log_file
     fi
     if [ "$LDAP_USER_BASEDN" = "" ]; 
     then
         DISABLE_LDAP=yes
         echo [`date`] LDAP user base DN is not defined. Skipping collect.>> $act_log_file
     fi
     if [ "$LDAP_GROUP_BASEDN" = "" ]; 
     then
         DISABLE_LDAP=yes
         echo [`date`] LDAP group base DN is not defined. Skipping collect.>> $act_log_file
     fi
     if [ "$LDAP_USERID_ATTRIBUTE" = "" ]; 
     then
         DISABLE_LDAP=yes
         echo [`date`] LDAP userID attribute is not defined. Skipping collect.>> $act_log_file
     fi
     if [ "$LDAP_AUTHID_ATTRIBUTE" = "" ]; 
     then
         DISABLE_LDAP=yes
         echo [`date`] LDAP authID attribute is not defined. Skipping collect.>> $act_log_file
     fi
     if [ "$LDAP_GROUPNAME_ATTRIBUTE" = "" ]; 
     then
         DISABLE_LDAP=yes
         echo [`date`] LDAP groupname attribute is not defined. Skipping collect.>> $act_log_file
     fi
     if [ "$LDAP_SEARCH_DN" = "" ]; 
     then
         DISABLE_LDAP=yes
         echo [`date`] LDAP login is not defined. Skipping collect.>> $act_log_file
     fi
     if [ "$LDAP_SEARCH_PW" = "" ]; 
     then
         DISABLE_LDAP=yes
         echo [`date`] LDAP password is not defined. Skipping collect.>> $act_log_file
     fi
     if [ "$LDAP_GROUP_LOOKUP_METHOD" = "" ]; 
     then
         DISABLE_LDAP=yes
         echo [`date`] LDAP group lookup method is not defined. Skipping collect.>> $act_log_file
     fi
     if [ "$LDAP_GROUP_LOOKUP_ATTRIBUTE" = "" ]; 
     then
         DISABLE_LDAP=yes
         echo [`date`] LDAP group lookup attribute is not defined. Skipping collect.>> $act_log_file
     fi
     
     if [ "$DISABLE_LDAP" = "no" ]; 
     then
         "$JAVA_HOME/bin/java"  -classpath "../bin/gettspmgroups.jar" nl.consul.cea.gensub.itspmuis.java.CollectTspmLdapGroups -LDAPHost?"$LDAP_HOST" -LDAPPort?"$LDAP_PORT" -UserName?"$LDAP_SEARCH_DN" -Password?"$LDAP_SEARCH_PW" -USER_BASEDN?"$LDAP_USER_BASEDN" -AUTHID_ATTRIBUTE?"$LDAP_AUTHID_ATTRIBUTE" -USER_OBJECTCLASS?"$LDAP_USER_OBJECTCLASS" -GROUP_OBJECTCLASS?"$LDAP_GROUP_OBJECTCLASS" -GROUP_BASEDN?"$LDAP_GROUP_BASEDN" -GROUPNAME_ATTRIBUTE?"$LDAP_GROUPNAME_ATTRIBUTE" -GROUP_LOOKUP_METHOD?"$LDAP_GROUP_LOOKUP_METHOD" -GROUP_LOOKUP_ATTRIBUTE?"$LDAP_GROUP_LOOKUP_ATTRIBUTE" -Filename?"$TEMP_FILE.ldap" 2>>$act_log_file 1>>$act_log_file
         if [ ! $? -eq 0 ] ; then
          echo [`date`] java run error. Failed to collect LDAP information.>> $act_log_file
         else
          echo [`date`] LDAP information collect is finished. >> $act_log_file
         fi
     fi 

     if [ -r "$TEMP_FILE.ldap" ] ; 
     then 
        cat "$TEMP_FILE.ldap" >> $TEMP_FILE
        rm $TEMP_FILE.ldap
     fi
else
    echo [`date`] LDAP host is not defined - skipping collect from LDAP.>> $act_log_file     
fi

# collecting DB2
if [ ! "$DB_HOST" = "" ]; 
then
     echo [`date`] Starting DB2 information collect. >> $act_log_file
     if [ "$DB_PORT" = "" ]; 
     then
         DISABLE_DB=yes
         echo [`date`] DB2 port is not defined. Skipping collect.>> $act_log_file
     fi
     if [ "$DB_USERNAME" = "" ]; 
     then
         DISABLE_DB=yes
         echo [`date`] DB2 username is not defined. Skipping collect.>> $act_log_file
     fi
     if [ "$DB_PASSWORD" = "" ]; 
     then
         DISABLE_DB=yes
         echo [`date`] DB2 password is not defined. Skipping collect.>> $act_log_file
     fi
     if [ "$DB_NAME" = "" ]; 
     then
         DISABLE_DB=yes
         echo [`date`] DB2 database name is not defined. Skipping collect.>> $act_log_file
     fi
     if [ "$DB_SCHEMA" = "" ]; 
     then
         DISABLE_DB=yes
         echo [`date`] DB2 database schema is not defined. Skipping collect.>> $act_log_file
     fi
          
     if [ "$DISABLE_DB" = "no" ]; 
     then
         echo [`date`] Collecting groups within administrative roles. >> $act_log_file
         echo "SELECT $DB_SCHEMA.ADMINROLE.NAME, $DB_SCHEMA.ADMINGROUP.REALM_UNIQUE_SECURITY_NAME FROM $DB_SCHEMA.ADMINROLE, $DB_SCHEMA.ADMINROLE_GROUPS, $DB_SCHEMA.ADMINGROUP WHERE $DB_SCHEMA.ADMINROLE.ID=$DB_SCHEMA.ADMINROLE_GROUPS.ID_OID AND $DB_SCHEMA.ADMINGROUP.ID=$DB_SCHEMA.ADMINROLE_GROUPS.ID_EID" > $PROPS_FILE.sql_groups
         "$JAVA_HOME/bin/java" -Dfile.encoding=UTF-8 -classpath "$CLASSPATH:../bin/jdbc-genact.jar" eprise.audit.chunktool.generic.java.JdbcGenact -driver=DB2 -host="$DB_HOST" -port="$DB_PORT" -database="$DB_NAME" -username="$DB_USERNAME" -passwd="$DB_PASSWORD" -props="$PROPS_FILE.tmp" -outfilename="$TEMP_FILE.db_groups" -sqlfilename="$PROPS_FILE.sql_groups" -useheaderline=no  -separator=";" -eraseoutfile=yes 2>>$act_log_file 1>>$act_log_file
         if [ ! $? -eq 0 ] ; then
          echo [`date`] java run error. Failed to collect DB information.>> $act_log_file
         else
          echo [`date`] DB information collect is finished. >> $act_log_file
         fi
         
         if [ -r "$TEMP_FILE.db_groups" ] ;
         then
             echo "[SourceGroup]" >> $TEMP_FILE
             # sed is used to convert windows-like line terminators to a unix-like
                          export IFS='
'
             for x in `cat $TEMP_FILE.db_groups | sed 's/.$//'`; do
                a=`echo $x | cut -d ";" -f 2`
                b=`echo $x | cut -d ";" -f 3`
                echo "\"$a role\"" >> $TEMP_FILE
                echo { >> $TEMP_FILE
                echo "   in src_group \"$b\"" >> $TEMP_FILE
                echo } >> $TEMP_FILE  
             done
             export IFS=''
         fi
         
         echo [`date`] Collecting users within administrative roles. >> $act_log_file
         echo "SELECT $DB_SCHEMA.ADMINROLE.NAME, $DB_SCHEMA.ADMINUSER.REALM_UNIQUE_SECURITY_NAME FROM $DB_SCHEMA.ADMINROLE, $DB_SCHEMA.ADMINROLE_USERS, $DB_SCHEMA.ADMINUSER WHERE $DB_SCHEMA.ADMINROLE.ID=$DB_SCHEMA.ADMINROLE_USERS.ID_OID AND $DB_SCHEMA.ADMINUSER.ID=$DB_SCHEMA.ADMINROLE_USERS.ID_EID" > $PROPS_FILE.sql_users
         "$JAVA_HOME/bin/java" -Dfile.encoding=UTF-8 -classpath "$CLASSPATH:../bin/jdbc-genact.jar" eprise.audit.chunktool.generic.java.JdbcGenact -driver=DB2 -host="$DB_HOST" -port="$DB_PORT" -database="$DB_NAME" -username="$DB_USERNAME" -passwd="$DB_PASSWORD" -props="$PROPS_FILE.tmp" -outfilename="$TEMP_FILE.db_users" -sqlfilename="$PROPS_FILE.sql_users" -useheaderline=no  -separator=";" -eraseoutfile=yes 2>>$act_log_file 1>>$act_log_file
         if [ ! $? -eq 0 ] ; then
          echo [`date`] java run error. Failed to collect DB information.>> $act_log_file
         else
          echo [`date`] DB information collect is finished. >> $act_log_file
         fi
         
         if [ -r "$TEMP_FILE.db_users" ] ;
         then
             echo "[SourceGroup]" >> $TEMP_FILE
             # sed is used to convert windows-like line terminators to a unix-like
                          export IFS='
'
             for x in `cat $TEMP_FILE.db_users | sed 's/.$//'`; do
                a=`echo $x | cut -d ";" -f 2`
                b=`echo $x | cut -d ";" -f 3`
                echo "\"$a role\"" >> $TEMP_FILE
                echo { >> $TEMP_FILE
                echo "   match logonname \"$b\"" >> $TEMP_FILE
                echo } >> $TEMP_FILE  
             done
             export IFS=''
         fi
         
         echo [`date`] Resolving user ID\'s from DB. >> $act_log_file
         echo SELECT DN, NAME FROM $DB_SCHEMA.\"USER\" > $PROPS_FILE.sql_uid
         "$JAVA_HOME/bin/java" -Dfile.encoding=UTF-8 -classpath "$CLASSPATH:../bin/jdbc-genact.jar" eprise.audit.chunktool.generic.java.JdbcGenact -driver=DB2 -host="$DB_HOST" -port="$DB_PORT" -database="$DB_NAME" -username="$DB_USERNAME" -passwd="$DB_PASSWORD" -props="$PROPS_FILE.tmp" -outfilename="$TEMP_FILE.db.uid" -sqlfilename="$PROPS_FILE.sql_uid" -useheaderline=no  -separator=";" -eraseoutfile=yes 2>>$act_log_file 1>>$act_log_file
         if [ ! $? -eq 0 ] ; then
          echo [`date`] java run error. Failed to collect DB information.>> $act_log_file
         else
          echo [`date`] DB information collect is finished. >> $act_log_file
         fi
         if [ -r "$TEMP_FILE.db.uid" ] ;
         then
             echo "[Persons]" >> $TEMP_FILE
             # sed is used to convert windows-like line terminators to a unix-like
             export IFS='
'
             for x in `cat $TEMP_FILE.db.uid | sed 's/.$//'`; do
                a=`echo $x | cut -d ";" -f 2`
                b=`echo $x | cut -d ";" -f 3`
                echo "\"$b\" \"$b\" \"0\"" >> $TEMP_FILE
                echo { >> $TEMP_FILE
                echo "   match logonname \"$a\"" >> $TEMP_FILE
                echo } >> $TEMP_FILE  
             done
             export IFS=''
         fi
         
         # clearing temporary files
         rm -f $TEMP_FILE.db.uid $PROPS_FILE.tmp $PROPS_FILE.sql_uid $TEMP_FILE.db_users $PROPS_FILE.sql_users $PROPS_FILE.sql_groups $TEMP_FILE.db_groups
         
     fi      
else
    echo [`date`] DB host is not defined - skipping collect from DB.>> $act_log_file
fi

echo "[`date`] Processing finished. [SUCCESS]">>$act_log_file;