rem IBM Confidential
rem OCO Source Material
rem 5724-S67
rem (c) Copyright IBM Corp. 1998, 2007
rem The source code for this program is not published or otherwise divested of
rem its trade secret, irrespective of what has been deposited with the U.S.
rem Copyright Office.

@echo off
rem #################################### gettsmplogs.bat #################################
rem Receives parameters from the TSIEM actuator
rem ######################################################################################
setlocal
set RTSS_LOGSET=%~1
set TSPM_LOGSET=%~2
set RTSS_LOG_LOCATION=%~3
set TSPM_LOG_LOCATION=%~4
set RTSS_LOG_TMPL=%~5
set TSPM_LOG_TMPL=%~6
set USER_NAME=%~7
set USER_PWD=%~8
for %%I in (%*) do set PROPS_FILE=%%~I
for %%I in (%PROPS_FILE%) do set PROPS_NAME=%%~nI

rem -- sets actuator log name
if "%PROPS_FILE%" == "" (
   set ACTUATOR_LOG="..\log\gettsmplogs.err"
) else (
   set ACTUATOR_LOG="..\log\%PROPS_NAME%.log"
)

rem ######################################################################################
rem TSIEM actuator parameters check
rem ######################################################################################
set PRINTING=enabled

rem -- one or both logset files arguments are missing
set LOGSETNAME_EMPTY=0
if "%RTSS_LOGSET%" == "" (
   call :print_message "ERROR: Missing output file name argument for logset of RTSS audit logs." %ACTUATOR_LOG%
   set LOGSETNAME_EMPTY=1
) else (
   type nul > "%RTSS_LOGSET%"
)
if "%TSPM_LOGSET%" == "" (
   call :print_message "ERROR: Missing output file name argument for logset of TSPM audit logs." %ACTUATOR_LOG%
   set LOGSETNAME_EMPTY=1
) else (
   type nul > "%TSPM_LOGSET%"
)
if %LOGSETNAME_EMPTY% EQU 1 goto failure

rem -- clears log file and start logging
type nul > %ACTUATOR_LOG%
rem -- Property file PROPS_FILE argument is missing
if "%PROPS_FILE%" == "" (
   call :print_message "ERROR: Missing property file argument." %ACTUATOR_LOG%
   goto failure
)
if not exist "%PROPS_FILE%" (
   call :print_message "ERROR: Can't find property file '%PROPS_FILE%'." %ACTUATOR_LOG%
   goto failure
)
call :print_message "Processing started." %ACTUATOR_LOG%

rem -- all arguments are present
set RTSS_COLLECT=1
if "%RTSS_LOG_LOCATION%" == "" (
   call :print_message "WARNING: Missing log directory path argument for RTSS audit log." %ACTUATOR_LOG%
   set RTSS_COLLECT=0
)
if "%RTSS_LOG_TMPL%" == "" (
   call :print_message "WARNING: Missing template of the log file name argument for RTSS audit log." %ACTUATOR_LOG%
   set RTSS_COLLECT=0
)
if %RTSS_COLLECT% EQU 0 (
   call :print_message "WARNING: RTSS audit log cannot be processed due to missing required parameter." %ACTUATOR_LOG%
)
set TSPM_COLLECT=1
if "%TSPM_LOG_LOCATION%" == "" (
   call :print_message "WARNING: Missing log directory path name argument for TSPM audit log." %ACTUATOR_LOG%
   set TSPM_COLLECT=0
)
if "%TSPM_LOG_TMPL%" == "" (
   call :print_message "WARNING: Missing template of the log file name argument for TSPM audit log." %ACTUATOR_LOG%
   set TSPM_COLLECT=0
)
if %TSPM_COLLECT% EQU 0 (
   call :print_message "WARNING: TSPM audit log cannot be processed due to missing required parameter." %ACTUATOR_LOG%
)
if "%RTSS_COLLECT%%TSPM_COLLECT%" == "00" (
   call :print_message "ERROR: Collect process cannot be completed due to previous errors." %ACTUATOR_LOG%
   goto failure
)

call :parsepropsfile
if "%HOST_NAME%" == "" (
   call :print_message "ERROR: 'SystemName' variable is not found in '%PROPS_FILE%' file." %ACTUATOR_LOG%
   goto failure
)

set TSIEM_BIN_DIR=%~dp0%
set TSIEM_BIN_DIR=%TSIEM_BIN_DIR:~0,-1%
set PS_EXE=%TSIEM_BIN_DIR%\psexec.exe
set PS_OPTIONS=-u %USER_NAME% -p %USER_PWD% -q
set NET_OPTIONS=%USER_PWD% /u:%USER_NAME%
set WORK_TMP_DIR=%SystemRoot%\Temp\ES-%PROPS_NAME%
set COLLECT_SCRIPT=eventscollect.bat
set COLLECT_PROGRESS_FILE=collectprogress.txt
set TIMESTAMP_FILE=timestamp.txt
set COLLECT_ERROR_FILE=collecterror.txt
set INITIAL_COLLECT_DAYS=2
set LOCREMDEC_SCRIPT=%TSIEM_BIN_DIR%\localorremote.vbs
set JAVA_BIN=java
set JAR_FILE=gettspmlogs.jar
set JAVA_CLASS=nl.consul.cea.gensub.itspm.java.ParseFile
set RTSS_TMP_LOGSET=rtss_logset
set TSPM_TMP_LOGSET=tspm_logset

Rem -- creates temp directory
call :print_message "Create temporary working directory '%WORK_TMP_DIR%'." %ACTUATOR_LOG%
if exist "%WORK_TMP_DIR%" rmdir /S /Q "%WORK_TMP_DIR%" >nul
mkdir "%WORK_TMP_DIR%" >nul

call :checkremotecollect || goto failure

if %REMOTE_COLLECT% EQU 1 (
   if "%USER_NAME%" == "" (
      call :print_message "ERROR: Missing username for accessing target system." %ACTUATOR_LOG%
      goto failure
   )
   if "%USER_PWD%" == "" (
      call :print_message "ERROR: Missing password for accessing target system." %ACTUATOR_LOG%
      goto failure
   )
)

call :print_message "- Start preparing environment for the collect process." %ACTUATOR_LOG%

call :createcollectscript

if %REMOTE_COLLECT% EQU 0 goto localcollect
   call :print_message "Find free drive letter for mapping network share." %ACTUATOR_LOG%
   call :findfreedisk || goto failure
   set COLLECT_DIR=%SHARE_DISK%\Temp\ES-%PROPS_NAME%

   call :print_message "Map network share of the target system on the drive '%SHARE_DISK%'." %ACTUATOR_LOG%
   net use %SHARE_DISK% /delete >nul
   net use %SHARE_DISK% \\%HOST_NAME%\ADMIN$ %NET_OPTIONS% >nul
   if not exist "%SHARE_DISK%\" (
      call :print_message "ERROR: Failure during mapping network share of the target system." %ACTUATOR_LOG%
      goto failure
   )

   if exist "%COLLECT_DIR%" rmdir /S /Q "%COLLECT_DIR%" >nul
   mkdir "%COLLECT_DIR%" >nul

   call :print_message "Copy necessary execution files to the target system." %ACTUATOR_LOG%
   rem -- Copying collect script to the target system
   call :copy2target "%WORK_TMP_DIR%" "%COLLECT_SCRIPT%" || goto failure

   rem -- Copying necessary binary files to the target system
   call :copy2target "%TSIEM_BIN_DIR%" "%JAR_FILE%" || goto failure

   rem -- Copying date.exe for getting timestamp on target system
   if not "%RTSS_INITIAL_COLLECT%%TSPM_INITIAL_COLLECT%" == "00" (
      call :copy2target "%TSIEM_BIN_DIR%" "date.exe" || goto failure
   )

   rem -- Flag message for catching error during starting collect script
   type nul > "%COLLECT_DIR%\%COLLECT_ERROR_FILE%"

   call :print_message "- Start the collect process." %ACTUATOR_LOG%
   rem -- Start collect script on target system and getting result files from target system
   %PS_EXE% \\%HOST_NAME% %PS_OPTIONS% cmd.exe /c "%%SystemRoot%%\Temp\ES-%PROPS_NAME%\%COLLECT_SCRIPT%">> %ACTUATOR_LOG%

   rem -- Copy collect progress log to main logfile
   call :print2log "%COLLECT_DIR%\%COLLECT_PROGRESS_FILE%"

   rem -- Check for collect script errors on target system
   if exist "%COLLECT_DIR%\%COLLECT_ERROR_FILE%" goto collecterror

   rem -- Get new timestamp from target system
   if exist "%COLLECT_DIR%\%TIMESTAMP_FILE%" (
      copy /Y "%COLLECT_DIR%\%TIMESTAMP_FILE%" "%WORK_TMP_DIR%\%TIMESTAMP_FILE%" >nul
      if errorlevel 1 (
         call :print_message "ERROR: Failure to get new timestamp from the target system." %ACTUATOR_LOG%
         goto failure
      )
   ) else (
      call :print_message "ERROR: There is not exist new timestamp on the target system." %ACTUATOR_LOG%
      goto failure
   )
   if "%RTSS_COLLECT%" == "1" (
      copy /B /Y "%COLLECT_DIR%\%RTSS_TMP_LOGSET%" "%RTSS_LOGSET%" >nul
      if errorlevel 1 (
         call :print_message "ERROR: Failure to copy RTSS collected logset from the target system." %ACTUATOR_LOG%
         goto failure
      )
   )
   if "%TSPM_COLLECT%" == "1" (
      copy /B /Y "%COLLECT_DIR%\%TSPM_TMP_LOGSET%" "%TSPM_LOGSET%" >nul
      if errorlevel 1 (
         call :print_message "ERROR: Failure to copy TSPM collected logset from the target system." %ACTUATOR_LOG%
         goto failure
      )
   )
   if exist "%COLLECT_DIR%" rmdir /S /Q "%COLLECT_DIR%" >nul
   net use %SHARE_DISK% /delete >nul
   goto finish

:localcollect
   set COLLECT_DIR=%WORK_TMP_DIR%

   call :print_message "- Start the collect process." %ACTUATOR_LOG%
   rem -- Start collect script
   call "%COLLECT_DIR%\%COLLECT_SCRIPT%"

   rem -- Check for collect script errors
   if exist "%COLLECT_DIR%\%COLLECT_ERROR_FILE%" goto collecterror
:finish

call :print_message "- Start postprocessing stage." %ACTUATOR_LOG%
rem -- Calculate amount of collected events
call :print_message "Calculate the amount of the collected events." %ACTUATOR_LOG%
for /f "tokens=*" %%I in ('find /C "<CommonBaseEvent " "%RTSS_LOGSET%" 2^>nul') do set EVENTS_NUMBER=%%I
for %%I in (%EVENTS_NUMBER%) do set EVENTS_NUMBER=%%I
if "%RTSS_COLLECT%" == "1" (
   call :print_message "The total number of new collected events from RTSS log is %EVENTS_NUMBER%." %ACTUATOR_LOG%
)
for /f "tokens=*" %%I in ('find /C "<CommonBaseEvent " "%TSPM_LOGSET%" 2^>nul') do set EVENTS_NUMBER=%%I
for %%I in (%EVENTS_NUMBER%) do set EVENTS_NUMBER=%%I
if "%TSPM_COLLECT%" == "1" (
   call :print_message "The total number of new collected events from TSPM log is %EVENTS_NUMBER%." %ACTUATOR_LOG%
)

call :print_message "Save the beginning timestamps for the next collect in the props-file." %ACTUATOR_LOG%
type "%WORK_TMP_DIR%\%TIMESTAMP_FILE%" > "%PROPS_FILE%"

call :cleanup
call :print_message "Processing finished [SUCCESS]." %ACTUATOR_LOG%
exit /b 0

rem -- Collect script finished with error
:collecterror

if %REMOTE_COLLECT% EQU 1 (
   call :print2log "%COLLECT_DIR%\%COLLECT_ERROR_FILE%"
) else (
   type "%COLLECT_DIR%\%COLLECT_ERROR_FILE%" >> %ACTUATOR_LOG%
)
call :print_message "ERROR: Failure during execution collect script." %ACTUATOR_LOG%

rem -- prints failure message to actuator log and finishes the script with errorlevel 1 (FAILURE)
:failure

call :cleanup
call :print_message "Processing finished [FAILURE]." %ACTUATOR_LOG%
exit /b 1

rem #########################################################################################
rem Parsing properties file PROPS_FILE for setting variables
rem #########################################################################################

:parsepropsfile

rem -- read value of the 'SystemName' parameter into HOST_NAME
for /f "tokens=2 delims==" %%I in ('findstr /R /C:"^SystemName = .*" "%PROPS_FILE%" 2^>nul') do set HOST_NAME=%%I
for /f %%I in ("%HOST_NAME%") do set HOST_NAME=%%I
rem -- read values of the timestamp and id parameters
for /f "tokens=2 delims==" %%I in ('findstr /R /C:"^RTSS_TIMESTAMP = .*" "%PROPS_FILE%" 2^>nul') do set RTSS_TIMESTAMP=%%I
for /f %%I in ("%RTSS_TIMESTAMP%") do set RTSS_TIMESTAMP=%%I
for /f "tokens=2 delims==" %%I in ('findstr /R /C:"^TSPM_TIMESTAMP = .*" "%PROPS_FILE%" 2^>nul') do set TSPM_TIMESTAMP=%%I
for /f %%I in ("%TSPM_TIMESTAMP%") do set TSPM_TIMESTAMP=%%I
for /f "tokens=2 delims==" %%I in ('findstr /R /C:"^RTSS_ID = .*" "%PROPS_FILE%" 2^>nul') do set RTSS_ID=%%I
for /f %%I in ("%RTSS_ID%") do set RTSS_ID=%%I
for /f "tokens=2 delims==" %%I in ('findstr /R /C:"^TSPM_ID = .*" "%PROPS_FILE%" 2^>nul') do set TSPM_ID=%%I
for /f %%I in ("%TSPM_ID%") do set TSPM_ID=%%I

set RTSS_INITIAL_COLLECT=1
if "%RTSS_TIMESTAMP%" NEQ "" (
   if "%RTSS_ID%" NEQ "" (
      set RTSS_INITIAL_COLLECT=0
   )
)
if %RTSS_COLLECT% EQU 1 (
   if "%RTSS_INITIAL_COLLECT%" == "1" (
      call :print_message "Initial collect mode use for RTSS log." %ACTUATOR_LOG%
   )
) else (
   set RTSS_INITIAL_COLLECT=0
)

set TSPM_INITIAL_COLLECT=1
if "%TSPM_TIMESTAMP%" NEQ "" (
   if "%TSPM_ID%" NEQ "" (
      set TSPM_INITIAL_COLLECT=0
   )
)
if %TSPM_COLLECT% EQU 1 (
   if "%TSPM_INITIAL_COLLECT%" == "1" (
      call :print_message "Initial collect mode use for TSPM log." %ACTUATOR_LOG%
   )
) else (
   set TSPM_INITIAL_COLLECT=0
)
exit /b


rem #########################################################################################
rem Check collecting mode and set REMOTE_COLLECT variable
rem #########################################################################################

:checkremotecollect

for /f "delims=" %%I in ('cscript.exe //Nologo "%LOCREMDEC_SCRIPT%" %HOST_NAME% 2^>nul') do set RESULT=%%I
for /f "tokens=1,2 delims=|" %%a in ("%RESULT%") do set RESULT1=%%a&set RESULT2=%%b

if "%RESULT1%" == "LOCAL" (
   set REMOTE_COLLECT=0
   call :print_message "Collect from the local target system: '%HOST_NAME%'." %ACTUATOR_LOG%
) else (
  if "%RESULT1%" == "REMOTE" (
     set REMOTE_COLLECT=1
     set HOST_NAME=%RESULT2%
     call :print_message "Collect from the remote target system: '%HOST_NAME%'." %ACTUATOR_LOG%
  ) else (
     call :print_message "ERROR: Script for determining mode of the collect return failure." %ACTUATOR_LOG%
     call :print_message "%RESULT2%" %ACTUATOR_LOG%
     exit /b 1
  )
)
exit /b


rem #########################################################################################
rem Creating collect script COLLECT_SCRIPT in WORK_TMP_DIR
rem #########################################################################################

:createcollectscript

set NEW_BAT_FILE="%WORK_TMP_DIR%\%COLLECT_SCRIPT%"
type nul > %NEW_BAT_FILE%

echo @echo off>> %NEW_BAT_FILE%
echo rem #################################### eventscollect.bat ####################################>> %NEW_BAT_FILE%
echo rem ### on success put new collected events from RTSS and TSPM logs into logsets files>> %NEW_BAT_FILE%
echo rem ### if failure return code 1 and explanation message in COLLECT_ERROR_FILE>> %NEW_BAT_FILE%
echo rem ###########################################################################################>> %NEW_BAT_FILE%
echo setlocal>> %NEW_BAT_FILE%
if %REMOTE_COLLECT% EQU 1 (
   echo ^(set WORK_TMP_DIR=%%SystemRoot%%\Temp\ES-%PROPS_NAME%^)>> %NEW_BAT_FILE%
   echo ^(set TSIEM_BIN_DIR=%%WORK_TMP_DIR%%^)>> %NEW_BAT_FILE%
   echo ^(set COLLECT_PROGRESS_FILE=^"%%WORK_TMP_DIR%%\%COLLECT_PROGRESS_FILE%^"^)>> %NEW_BAT_FILE%
   echo ^(set RTSS_LOGSET=%%WORK_TMP_DIR%%\%RTSS_TMP_LOGSET%^)>> %NEW_BAT_FILE%
   echo ^(set TSPM_LOGSET=%%WORK_TMP_DIR%%\%TSPM_TMP_LOGSET%^)>> %NEW_BAT_FILE%
   echo type nul ^> ^"%%RTSS_LOGSET%%^">> %NEW_BAT_FILE%
   echo type nul ^> ^"%%TSPM_LOGSET%%^">> %NEW_BAT_FILE%
) else (
   echo ^(set WORK_TMP_DIR=%WORK_TMP_DIR%^)>> %NEW_BAT_FILE%
   echo ^(set TSIEM_BIN_DIR=%TSIEM_BIN_DIR%^)>> %NEW_BAT_FILE%
   echo ^(set COLLECT_PROGRESS_FILE=%ACTUATOR_LOG%^)>> %NEW_BAT_FILE%
   echo ^(set RTSS_LOGSET=%RTSS_LOGSET%^)>> %NEW_BAT_FILE%
   echo ^(set TSPM_LOGSET=%TSPM_LOGSET%^)>> %NEW_BAT_FILE%
)
echo ^(set TIMESTAMP_FILE=^"%%WORK_TMP_DIR%%\%TIMESTAMP_FILE%^"^)>> %NEW_BAT_FILE%
echo ^(set COLLECT_ERROR_FILE=^"%%WORK_TMP_DIR%%\%COLLECT_ERROR_FILE%^"^)>> %NEW_BAT_FILE%
echo ^(set RTSS_LOG_LOCATION=%RTSS_LOG_LOCATION%^)>> %NEW_BAT_FILE%
echo ^(set TSPM_LOG_LOCATION=%TSPM_LOG_LOCATION%^)>> %NEW_BAT_FILE%
echo ^(set RTSS_LOG_TMPL=%RTSS_LOG_TMPL%^)>> %NEW_BAT_FILE%
echo ^(set TSPM_LOG_TMPL=%TSPM_LOG_TMPL%^)>> %NEW_BAT_FILE%
echo ^(set RTSS_INITIAL_COLLECT=%RTSS_INITIAL_COLLECT%^)>> %NEW_BAT_FILE%
echo ^(set TSPM_INITIAL_COLLECT=%TSPM_INITIAL_COLLECT%^)>> %NEW_BAT_FILE%
echo ^(set RESULT_CODE=^)>> %NEW_BAT_FILE%
echo if exist %%COLLECT_ERROR_FILE%% del /F /Q %%COLLECT_ERROR_FILE%% ^>nul>> %NEW_BAT_FILE%
echo for /F ^"tokens=2,3 delims=:^" %%%%A in ^(^'where ^"%JAVA_BIN%^" 2^^^>nul ^^^| findstr /N ^"%JAVA_BIN%^" 2^^^>nul ^^^| findstr /R ^"^^1:^" 2^^^>nul^'^) do set JAVA_PATH=%%%%A:%%%%B>> %NEW_BAT_FILE%
echo if ^"%%JAVA_PATH%%^" == ^"^" ^(>> %NEW_BAT_FILE%
echo    call :print_message ^"ERROR: Java binary file not found.^" %%COLLECT_ERROR_FILE%%>> %NEW_BAT_FILE%
echo    exit /b ^1>> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
echo call :print_message ^"Found Java binary file: ^'%%JAVA_PATH%%^'.^" %%COLLECT_PROGRESS_FILE%%>> %NEW_BAT_FILE%
if not "%RTSS_INITIAL_COLLECT%%TSPM_INITIAL_COLLECT%" == "00" (
   echo call :print_message ^"Calculate the timestamp, from which the events will be collected.^" %%COLLECT_PROGRESS_FILE%%>> %NEW_BAT_FILE%
   echo for /F ^"tokens=*^" %%%%I in ^(^'^"^"%%TSIEM_BIN_DIR%%\date.exe^" ^"+%%%%Y-%%%%m-%%%%dT%%%%H:%%%%M:%%%%S.000Z^" -d ^"%INITIAL_COLLECT_DAYS% days ago^"^" 2^^^>nul^'^) do set TIMESTAMP=%%%%I>> %NEW_BAT_FILE%
   echo if ^"%%TIMESTAMP%%^" == ^"^" ^(>> %NEW_BAT_FILE%
   echo     call :print_message ^"ERROR: Failure to get timestamp for initial collect for RTSS log.^" %%COLLECT_ERROR_FILE%%>> %NEW_BAT_FILE%
   echo     exit /b ^1>> %NEW_BAT_FILE%
   echo ^)>> %NEW_BAT_FILE%
)
if %RTSS_INITIAL_COLLECT% EQU 1 (
   echo ^(set RTSS_TIMESTAMP=%%TIMESTAMP%%^)>> %NEW_BAT_FILE%
   echo ^(set RTSS_ID=firstcollect^)>> %NEW_BAT_FILE%
) else (
   echo ^(set RTSS_TIMESTAMP=%RTSS_TIMESTAMP%^)>> %NEW_BAT_FILE%
   echo ^(set RTSS_ID=%RTSS_ID%^)>> %NEW_BAT_FILE%
)
if %TSPM_INITIAL_COLLECT% EQU 1 (
   echo ^(set TSPM_TIMESTAMP=%%TIMESTAMP%%^)>> %NEW_BAT_FILE%
   echo ^(set TSPM_ID=firstcollect^)>> %NEW_BAT_FILE%
) else (
   echo ^(set TSPM_TIMESTAMP=%TSPM_TIMESTAMP%^)>> %NEW_BAT_FILE%
   echo ^(set TSPM_ID=%TSPM_ID%^)>> %NEW_BAT_FILE%
)
if %RTSS_COLLECT% EQU 1 (
   echo call :collectlog RTSS>> %NEW_BAT_FILE%
) else (
   if %RTSS_INITIAL_COLLECT% EQU 0 (
      echo echo RTSS_TIMESTAMP=%RTSS_TIMESTAMP%^>^> ^"%%TIMESTAMP_FILE%%^">> %NEW_BAT_FILE%
      echo echo RTSS_ID=%RTSS_ID%^>^> ^"%%TIMESTAMP_FILE%%^">> %NEW_BAT_FILE%
   )
   echo ^(set RESULT_CODE=1^)>> %NEW_BAT_FILE%
)
if %TSPM_COLLECT% EQU 1 (
   echo call :collectlog TSPM>> %NEW_BAT_FILE%
) else (
   if %TSPM_INITIAL_COLLECT% EQU 0 (
      echo echo TSPM_TIMESTAMP=%TSPM_TIMESTAMP%^>^> ^"%%TIMESTAMP_FILE%%^">> %NEW_BAT_FILE%
      echo echo TSPM_ID=%TSPM_ID%^>^> ^"%%TIMESTAMP_FILE%%^">> %NEW_BAT_FILE%
   )
   echo ^(set RESULT_CODE=1^)>> %NEW_BAT_FILE%
)
echo if ^"%%RESULT_CODE%%^" EQU ^"11^" ^(>> %NEW_BAT_FILE%
echo    call :print_message ^"ERROR: Neither of required logs collected successfully.^" %%COLLECT_PROGRESS_FILE%%>> %NEW_BAT_FILE%
echo    exit /b ^1>> %NEW_BAT_FILE%
echo ^) else ^(>> %NEW_BAT_FILE%
echo    exit /b ^0>> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
echo rem #########################################################################################>> %NEW_BAT_FILE%
echo rem Collect events from required log file>> %NEW_BAT_FILE%
echo rem %%1  - log type ^(RTSS^|TSPM^)>> %NEW_BAT_FILE%
echo rem #########################################################################################>> %NEW_BAT_FILE%
echo :collectlog>> %NEW_BAT_FILE%
echo set LOG_LOCATION=%%1_LOG_LOCATION>> %NEW_BAT_FILE%
echo call set LOG_LOCATION=%%%%%%LOG_LOCATION%%%%%%>> %NEW_BAT_FILE%
echo set LOG_TMPL=%%1_LOG_TMPL>> %NEW_BAT_FILE%
echo call set LOG_TMPL=%%%%%%LOG_TMPL%%%%%%>> %NEW_BAT_FILE%
echo set TIMESTAMP=%%1_TIMESTAMP>> %NEW_BAT_FILE%
echo call set TIMESTAMP=%%%%%%TIMESTAMP%%%%%%>> %NEW_BAT_FILE%
echo set ID=%%1_ID>> %NEW_BAT_FILE%
echo call set ID=%%%%%%ID%%%%%%>> %NEW_BAT_FILE%
echo set LOGSET=%%1_LOGSET>> %NEW_BAT_FILE%
echo call set LOGSET=%%%%%%LOGSET%%%%%%>> %NEW_BAT_FILE%
echo set INITIAL=%%1_INITIAL_COLLECT>> %NEW_BAT_FILE%
echo call set INITIAL=%%%%%%INITIAL%%%%%%>> %NEW_BAT_FILE%
echo set NEWEVENTS_FILE=%%WORK_TMP_DIR%%\newevents.xml>> %NEW_BAT_FILE%
echo set LASTTIMESTAMP_FILE=%%WORK_TMP_DIR%%\lasttimestamp.txt>> %NEW_BAT_FILE%
echo type nul^>^"%%LASTTIMESTAMP_FILE%%^"
echo set FAILURE=^0>> %NEW_BAT_FILE%
echo set TIMESTAMP_ORIG=%%TIMESTAMP%%>> %NEW_BAT_FILE%
echo set ID_ORIG=%%ID%%>> %NEW_BAT_FILE%
echo call :print_message ^"*****************************^" %%COLLECT_PROGRESS_FILE%%>> %NEW_BAT_FILE%
echo call :print_message ^"* Start processing %%1 log *^" %%COLLECT_PROGRESS_FILE%%>> %NEW_BAT_FILE%
echo call :print_message ^"*****************************^" %%COLLECT_PROGRESS_FILE%%>> %NEW_BAT_FILE%
echo call :print_message ^"Events will be collected from:^" %%COLLECT_PROGRESS_FILE%%>> %NEW_BAT_FILE%
echo call :print_message ^" TIMESTAMP = %%TIMESTAMP%%, ID = %%ID%%^" %%COLLECT_PROGRESS_FILE%%>> %NEW_BAT_FILE%
echo set FILES_COUNT=^0>> %NEW_BAT_FILE%
echo for /F %%%%I in ^(^'dir ^"%%LOG_LOCATION%%^" /B /A-D /O-N ^^^| findstr /R /C:^"^^%%LOG_TMPL%%[0-9][0-9]*$^"^'^) do ^(>> %NEW_BAT_FILE%
echo     set /A FILES_COUNT+=^1>> %NEW_BAT_FILE%
echo     call :print_message ^"Process file ^'%%LOG_LOCATION%%\%%%%I^'.^" %%COLLECT_PROGRESS_FILE%%>> %NEW_BAT_FILE%
echo     if exist ^"%%NEWEVENTS_FILE%%^" del /Q /F ^"%%NEWEVENTS_FILE%%^" ^>nul>> %NEW_BAT_FILE%
echo     ^"%%JAVA_PATH%%^" -Dfile.encoding=UTF-8 -classpath ^"%%TSIEM_BIN_DIR%%\%JAR_FILE%^" ^"%JAVA_CLASS%^" ^"%%LOG_LOCATION%%\%%%%I^" ^"%%NEWEVENTS_FILE%%^" ^"%%TIMESTAMP%%^" cbe Windows ^"%%LASTTIMESTAMP_FILE%%^" ^"%%ID%%^"^>^> %%COLLECT_PROGRESS_FILE%%>> %NEW_BAT_FILE%
echo     if errorlevel 5 ^(>> %NEW_BAT_FILE%
echo        call :print_message ^"ERROR: Unrecognized exit code - %errorlevel%.^" %%COLLECT_PROGRESS_FILE%%>> %NEW_BAT_FILE%
echo        set FAILURE=^1>> %NEW_BAT_FILE%
echo     ^) else ^(>> %NEW_BAT_FILE%
echo        if errorlevel 4 ^(>> %NEW_BAT_FILE%
echo           call :print_message ^"No new events found.^" %%COLLECT_PROGRESS_FILE%%>> %NEW_BAT_FILE%
echo           call :setlastbound ^"%%LASTTIMESTAMP_FILE%%^">> %NEW_BAT_FILE%
echo           if ^"%%TIMESTAMP%%^" == ^"^" ^(>> %NEW_BAT_FILE%
echo              if %%INITIAL%% EQU 1 ^(>> %NEW_BAT_FILE%
echo                 for /F ^"tokens=*^" %%%%I in ^(^'^"^"%%TSIEM_BIN_DIR%%\date.exe^" ^"+%%%%Y-%%%%m-%%%%dT%%%%H:%%%%M:%%%%S.000Z^"^" 2^^^>nul^'^) do set TIMESTAMP=%%%%I>> %NEW_BAT_FILE%
echo              ^) else ^(>> %NEW_BAT_FILE%
echo                 set TIMESTAMP=%%TIMESTAMP_ORIG%%>> %NEW_BAT_FILE%
echo              ^)>> %NEW_BAT_FILE%
echo           ^)>> %NEW_BAT_FILE%
echo        ^) else ^(>> %NEW_BAT_FILE%
echo           if errorlevel 3 ^(>> %NEW_BAT_FILE%
echo              if exist ^"%%NEWEVENTS_FILE%%^" ^(>> %NEW_BAT_FILE%
echo                 copy /B /Y ^"%%LOGSET%%^" + ^"%%NEWEVENTS_FILE%%^" ^"%%LOGSET%%^" ^>nul>> %NEW_BAT_FILE%
echo                 call :setlastbound ^"%%LASTTIMESTAMP_FILE%%^">> %NEW_BAT_FILE%
echo              ^) else ^(>> %NEW_BAT_FILE%
echo                 set FAILURE=^1>> %NEW_BAT_FILE%
echo                 call :print_message ^"ERROR: Temporary file ^'%%NEWEVENTS_FILE%%^' not found.^" %%COLLECT_PROGRESS_FILE%%>> %NEW_BAT_FILE%
echo              ^)>> %NEW_BAT_FILE%
echo           ^) else ^(  >> %NEW_BAT_FILE%
echo              if errorlevel 2 ^(>> %NEW_BAT_FILE%
echo                 if exist ^"%%LOG_LOCATION%%\%%%%I^" ^(      >> %NEW_BAT_FILE%
echo                    copy /B /Y ^"%%LOGSET%%^" + ^"%%LOG_LOCATION%%\%%%%I^" ^"%%LOGSET%%^" ^>nul>> %NEW_BAT_FILE%
echo                    call :setlastbound ^"%%LASTTIMESTAMP_FILE%%^">> %NEW_BAT_FILE%
echo                 ^) else ^(>> %NEW_BAT_FILE%
echo                    set FAILURE=^1>> %NEW_BAT_FILE%
echo                    call :print_message ^"ERROR: Log file ^'%%LOG_LOCATION%%\%%%%I^' not found.^" %%COLLECT_PROGRESS_FILE%%>> %NEW_BAT_FILE%
echo                 ^)>> %NEW_BAT_FILE%
echo              ^) else ^(>> %NEW_BAT_FILE%
echo                 set FAILURE=^1>> %NEW_BAT_FILE%
echo                 if errorlevel 1 ^(>> %NEW_BAT_FILE%
echo                    call :print_message ^"ERROR: Supplied parameters are incorrect.^" %%COLLECT_PROGRESS_FILE%%>> %NEW_BAT_FILE%
echo                 ^) else ^(>> %NEW_BAT_FILE%
echo                    call :print_message ^"ERROR: Unrecognized exit code - %errorlevel%.^" %%COLLECT_PROGRESS_FILE%%>> %NEW_BAT_FILE%
echo                 ^)>> %NEW_BAT_FILE%
echo              ^)>> %NEW_BAT_FILE%
echo           ^)>> %NEW_BAT_FILE%
echo        ^)>> %NEW_BAT_FILE%
echo     ^)>> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
echo set SAVE_STATE=^1>> %NEW_BAT_FILE%
echo if %%FAILURE%% EQU 0 ^(>> %NEW_BAT_FILE%
echo    if %%FILES_COUNT%% EQU 0 ^(>> %NEW_BAT_FILE%
echo       call :print_message ^"WARNING: No %%1 log files found, which satisfy search criteria.^" %%COLLECT_PROGRESS_FILE%%>> %NEW_BAT_FILE%
echo       if %%INITIAL%% EQU 1 set SAVE_STATE=^0>> %NEW_BAT_FILE%
echo       set FAILURE=^1>> %NEW_BAT_FILE%
echo    ^) else ^(>> %NEW_BAT_FILE%
echo       call :print_message ^"Number of processed log files: %%FILES_COUNT%%.^" %%COLLECT_PROGRESS_FILE%%>> %NEW_BAT_FILE%
echo       call :print_message ^"Successfully finished processing %%1 log.^" %%COLLECT_PROGRESS_FILE%%>> %NEW_BAT_FILE%
echo    ^)>> %NEW_BAT_FILE%
echo ^) else ^(>> %NEW_BAT_FILE%
echo    set %%1_TIMESTAMP=%%TIMESTAMP_ORIG%%>> %NEW_BAT_FILE%
echo    set %%1_ID=%%ID_ORIG%%>> %NEW_BAT_FILE%
echo    call :print_message ^"[ERROR] Abort processing %%1 log.^" %%COLLECT_PROGRESS_FILE%%>> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
echo if %%SAVE_STATE%% EQU 1 ^(>> %NEW_BAT_FILE%
echo    call :print_message ^"Save agent state information for the next collect:^" %%COLLECT_PROGRESS_FILE%%>> %NEW_BAT_FILE%
echo    call :print_message ^" TIMESTAMP = %%TIMESTAMP%%, ID = %%ID%%^" %%COLLECT_PROGRESS_FILE%%>> %NEW_BAT_FILE%
echo    echo %%1_TIMESTAMP = %%TIMESTAMP%%^>^> ^"%%TIMESTAMP_FILE%%^">> %NEW_BAT_FILE%
echo    echo %%1_ID = %%ID%%^>^> ^"%%TIMESTAMP_FILE%%^">> %NEW_BAT_FILE%
echo ^)>> %NEW_BAT_FILE%
echo set RESULT_CODE=%%RESULT_CODE%%%%FAILURE%%>> %NEW_BAT_FILE%
echo exit /b>> %NEW_BAT_FILE%
echo rem #########################################################################################>> %NEW_BAT_FILE%
echo rem Set up agent state information from result file>> %NEW_BAT_FILE%
echo rem #########################################################################################>> %NEW_BAT_FILE%
echo :setlastbound>> %NEW_BAT_FILE%
echo for /F ^"tokens=2 delims==^" %%%%I in ^(^'findstr timestamp %%1^'^) do set TIMESTAMP=%%%%I>> %NEW_BAT_FILE%
echo for /F ^"tokens=2 delims==^" %%%%I in ^(^'findstr globalId %%1^'^) do set ID=%%%%I>> %NEW_BAT_FILE%
echo exit /b>> %NEW_BAT_FILE%
echo rem ######################################################################################>> %NEW_BAT_FILE%
echo rem printing routine>> %NEW_BAT_FILE%
echo rem prints a message to actuator log. >> %NEW_BAT_FILE%
echo rem tekes two parameters:  - message itself,  - name of a log file>> %NEW_BAT_FILE%
echo rem ######################################################################################>> %NEW_BAT_FILE%
echo :print_message>> %NEW_BAT_FILE%
echo echo [%%date%%][%%time%%] %%~1^>^> %%^2>> %NEW_BAT_FILE%
echo exit /b>> %NEW_BAT_FILE%

call :print_message "File '%COLLECT_SCRIPT%' generated." %ACTUATOR_LOG%
exit /b

rem ######################################################################################
rem printing routine
rem prints a message to actuator log. 
rem tekes two parameters: %1 - message itself, %2 - name of a log file
rem ######################################################################################

:print_message

if "%PRINTING%" == "enabled" @echo [%date%][%time%] %~1>> %2
exit /b

rem ######################################################################################
rem Searching for unused drive letter for mapping shared resource
rem ######################################################################################

:findfreedisk

for %%I in (Z Y X W V U T S R Q P O N M L K) do (
    if not exist "%%I:\" (
        cd %%I: > nul 2>&1
        if errorlevel 1 (
            set SHARE_DISK=%%I:
            exit /b 0
        )
    )
)

call :print_message "ERROR: Could not find unused drive letter for mapping target network share." %ACTUATOR_LOG%
exit /b 1


rem ######################################################################################
rem Copy content to logfile
rem ######################################################################################

:print2log

if exist "%~1" (
  for /f "delims=" %%I in ('type "%~1" 2^>nul') do (
    call :print_message "%HOST_NAME%: %%I" %ACTUATOR_LOG%
  )
)
exit /b


rem ######################################################################################
rem Copy files to the target system for execution
rem ######################################################################################

:copy2target

for %%I in (%~2) do (
    copy /Y "%~1\%%I" "%COLLECT_DIR%" >nul
    if errorlevel 1 (
       call :print_message "ERROR: Failure to copy '%%I' to the target system." %ACTUATOR_LOG%
       exit /b 1
    ) else (
       call :print_message "File '%%I' successfully copied to the target system." %ACTUATOR_LOG%
    )
)
exit /b 0


rem ######################################################################################
rem Cleanup routine. Deleting temp directories and unmaping share
rem ######################################################################################

:cleanup

call :print_message "Free and clean up all used system resources." %ACTUATOR_LOG%
if not "%COLLECT_DIR%" == "" (
   if exist "%COLLECT_DIR%" rmdir /S /Q "%COLLECT_DIR%" >nul
)
if not "%SHARE_DISK%" == "" (
   if exist "%SHARE_DISK%\" net use "%SHARE_DISK%" /delete >nul
)
if not "%WORK_TMP_DIR%" == "" (
   if exist "%WORK_TMP_DIR%" rmdir /S /Q "%WORK_TMP_DIR%" >nul
)
exit /b
