#!/bin/ksh
#gettspmlogs.sh 1tmp.tmp 2tmp.tmp ../temp ../temp rtss.audit.log tspm.audit.log me.props]

mounth2num () {   
  case $1 in
    'Jan')
       outval="01"
       ;;
    'Feb')
       outval="02"
       ;;
    'Mar')
       outval="03"
       ;;
    'Apr')
       outval="04"
       ;;
    'May')
       outval="05"
       ;;
    'Jun')
       outval="06"
       ;;
    'Jul')
       outval="07"
       ;;
    'Aug')
       outval="08"
       ;;
    'Sep')
       outval="09"
       ;;
    'Oct')
       outval="10"
       ;;
    'Nov')
       outval="11"
       ;;
    'Dec')
       outval="12"
       ;;
     *)
       outval="00"
       ;;
  esac
  echo $outval
} 
first_collect()
{
  DATE_NOW=`date +"%d-%b-%Y"`
  TIME_NOW=`date +"%H:%M"`
  DATE_THEN=`../bin/goBackInTime $DATE_NOW $TIME_NOW 2880` 

  DD=`echo $DATE_THEN | sed -e "s/\(.*\)-.*-.* .*:.*/\1/"`
  MON=`echo $DATE_THEN | sed -e "s/.*-\(.*\)-.* .*:.*/\1/"`
  MON=`mounth2num $MON`
  YYYY=`echo $DATE_THEN | sed -e "s/.*-.*-\(.*\) .*:.*/\1/"`
  HH=`echo $DATE_THEN | sed -e "s/.*-.*-.* \(.*\):.*/\1/"`
  MM=`echo $DATE_THEN | sed -e "s/.*-.*-.* .*:\(.*\)/\1/"`
#2009-04-10T11:07:02.798Z
echo $YYYY-$MON-$DD"T"$HH":"$MM":00.000Z"
}

setlastbound () {
  TIMESTAMP=`sed -n 's/^timestamp=\(.*\)$/\1/p' $1`
  ID=`sed -n 's/^globalId=\(.*\)$/\1/p' $1`
}


LOGSET=$1
LOG_LOCATION=$2
LOG_TMPL=$3
LOG_TYPE=$4

touch "$LOGSET"

TIMESTAMP=`sed -n 's/^'${LOG_TYPE}'_TIMESTAMP =[ 	]*\([^ 	]*\)[ 	]*$/\1/p' $PROPSFILE`
ID=`sed -n 's/^'${LOG_TYPE}'_ID =[ 	]*\([^ 	]*\)[ 	]*$/\1/p' $PROPSFILE`

[ -z "$LOG_LOCATION" ] && echo "[`date`] WARNING: Missing log directory path argument for ${LOG_TYPE} audit log." >> $ACTUATOR_LOG
[ -z "$LOG_TMPL" ] && echo "[`date`] WARNING: Missing template of the log file name argument for ${LOG_TYPE} audit log." >> $ACTUATOR_LOG

if [ -z "$LOG_LOCATION" -o -z "$LOG_TMPL" ]; then
   echo "[`date`] WARNING: ${LOG_TYPE} audit log cannot be processed due to missing required parameter." >> $ACTUATOR_LOG
   if [ ! -z "$TIMESTAMP" -a ! -z "$ID" ]; then
      echo "${LOG_TYPE}_TIMESTAMP=$TIMESTAMP" >> $PROPS_TEMP
      echo "${LOG_TYPE}_ID=$ID" >> $PROPS_TEMP
   fi
   exit 2
fi

if [ -f $JAVAPATH_FILE ]; then
   JAVA_PATH=`cat $JAVAPATH_FILE`
else
   JAVA_PATH=""
fi
if [ -z "$JAVA_PATH" -o ! \( -f "$JAVA_PATH" -a -x "$JAVA_PATH" \) ]; then
   JAVA_PATH=`which java`
   if [ -f "$JAVA_PATH" -a -x "$JAVA_PATH" ]; then
      echo "[`date`] Found Java binary file: '${JAVA_PATH}'." >> $ACTUATOR_LOG
      echo "$JAVA_PATH" > $JAVAPATH_FILE
   else
      echo "[`date`] ERROR: Java binary file not found." >> $ACTUATOR_LOG
      exit 3
   fi
fi

if [ -z "$TIMESTAMP" -o -z "$ID" ]; then
   echo "[`date`] Calculate the timestamp, from which the events will be collected." >> $ACTUATOR_LOG
   TIMESTAMP=`first_collect`
   ID="firstcollect"
   echo "[`date`] Initial collect mode use for ${LOG_TYPE} log." >> $ACTUATOR_LOG
   INITIAL_COLLECT=1
else
   INITIAL_COLLECT=0
fi

TIMESTAMP_ORIG=$TIMESTAMP
ID_ORIG=$ID

echo "[`date`] *****************************" >> $ACTUATOR_LOG
echo "[`date`] * Start processing ${LOG_TYPE} log *" >> $ACTUATOR_LOG
echo "[`date`] *****************************" >> $ACTUATOR_LOG
echo "[`date`] Events will be collected from:" >> $ACTUATOR_LOG
echo "[`date`]  TIMESTAMP = ${TIMESTAMP}, ID = ${ID}" >> $ACTUATOR_LOG

NEWEVENTS_FILE=newevents.xml
LASTTIMESTAMP_FILE=lasttimestamp.txt
FAILURE=0
FILES_COUNT=0
echo "" > "$LASTTIMESTAMP_FILE"
for LOGFILE in `ls -1 "$LOG_LOCATION" | grep "^${LOG_TMPL}[0-9][0-9]*$" | sort -r`; do
    FILES_COUNT=`expr $FILES_COUNT + 1`
    echo "[`date`] Process file '$LOG_LOCATION/$LOGFILE'." >> $ACTUATOR_LOG
    [ -f "$NEWEVENTS_FILE" ] && rm -f "$NEWEVENTS_FILE"
    "$JAVA_PATH" -classpath "../bin/gettspmlogs.jar" nl.consul.cea.gensub.itspm.java.ParseFile "$LOG_LOCATION/$LOGFILE" "$NEWEVENTS_FILE" "$TIMESTAMP" cbe UNIX "$LASTTIMESTAMP_FILE" "$ID"
    RESULT_CODE=$?
    if [ $RESULT_CODE -lt 2 -o $RESULT_CODE -gt 4 ]; then
       FAILURE=1
       if [ $RESULT_CODE -eq 1 ]; then
          echo "[`date`] ERROR: Supplied parameters are incorrect." >> $ACTUATOR_LOG
       else
          echo "[`date`] ERROR: Unrecognized exit code - ${RESULT_CODE}." >> $ACTUATOR_LOG
       fi
    else
       case $RESULT_CODE in
          2)
            if [ -f "$LOG_LOCATION/$LOGFILE" ]; then      
               cat "$LOG_LOCATION/$LOGFILE" >> "$LOGSET"
               setlastbound "$LASTTIMESTAMP_FILE"
            else
               FAILURE=1
               echo "[`date`] ERROR: Log file '${LOG_LOCATION}/${LOGFILE}' not found." >> $ACTUATOR_LOG
            fi
            ;;
          3)
            if [ -f "$NEWEVENTS_FILE" ]; then
               cat "$NEWEVENTS_FILE" >> "$LOGSET"
               setlastbound "$LASTTIMESTAMP_FILE"
            else
               FAILURE=1
               echo "[`date`] ERROR: Temporary file '$NEWEVENTS_FILE' not found." >> $ACTUATOR_LOG
            fi
            ;;
          4)
            echo "[`date`] No new events found." >> $ACTUATOR_LOG
            setlastbound "$LASTTIMESTAMP_FILE"
            if [ $INITIAL_COLLECT -eq 1 ]; then
               TIMESTAMP=`date "+%Y-%m-%dT%H:%M:%S.000Z"`
            else
               TIMESTAMP=$TIMESTAMP_ORIG
            fi
            ;;
       esac
    fi
done

SAVE_STATE=1
if [ $FAILURE -eq 0 ]; then
   if [ $FILES_COUNT -eq 0 ]; then
      echo "[`date`] WARNING: No ${LOG_TYPE} log files found, which satisfy search criteria." >> $ACTUATOR_LOG
      [ $INITIAL_COLLECT -eq 1 ] && SAVE_STATE=0
      FAILURE=1
   else
      echo "[`date`] Number of processed log files: $FILES_COUNT." >> $ACTUATOR_LOG
      echo "[`date`] Successfully finished processing ${LOG_TYPE} log." >> $ACTUATOR_LOG
   fi
else
   TIMESTAMP=$TIMESTAMP_ORIG
   ID=$ID_ORIG
   echo "[`date`] [ERROR] Abort processing ${LOG_TYPE} log." >> $ACTUATOR_LOG
fi
if [ $SAVE_STATE -eq 1 ]; then
   echo "[`date`] Save agent state information for the next collect:" >> $ACTUATOR_LOG
   echo "[`date`]  TIMESTAMP = ${TIMESTAMP}, ID = ${ID}" >> $ACTUATOR_LOG
   echo "${LOG_TYPE}_TIMESTAMP = $TIMESTAMP" >> $PROPS_TEMP
   echo "${LOG_TYPE}_ID = $ID" >> $PROPS_TEMP
fi

exit $FAILURE
