@echo off

set TEMP1=%~1
set TEMP2=%~2
set LOGDIR=%~3
set LOGNAME=%~4
set ARCHDIR=%~5
set JAVA_PATH=%~6
set SAP_LOG_API_PATH=%~7
for %%I in (%*) do set PROPS_FILE=%%~I
for %%I in (%PROPS_FILE%) do set PROPS_NAME=%%~nI

set JAR_FILE=../bin/getsapjslogs.jar
set JAVA_CLASS=nl.consul.cea.gensub.sapjs.java.GetSAPLogs
set JAVA_EXE=java.exe

if "%PROPS_FILE%" == "" (
   set ES_LOG="..\log\getsapjslogs-error.log"
   call :print_message "ERROR: Missing properties file argument"
   goto failure
) else (
   set ES_LOG="..\log\%PROPS_NAME%.log"
)
if not exist "%PROPS_FILE%" (
   call :print_message "ERROR: Properties file '%PROPS_FILE%' does not exist"
   goto failure
)

if "%TSIEM_TMP_DIR%" == "" (
   set CHUNKS_DIR=%TEMP%
) else (
   set CHUNKS_DIR=%TSIEM_TMP_DIR%
)
if "%CHUNKS_DIR%" == "" (
   set CHUNKS_DIR=%SystemRoot%\Temp
)
set CHUNKS_DIR=%CHUNKS_DIR%\ES-%PROPS_NAME%

type nul > %ES_LOG%
if "%TEMP1%" == "" (
   call :print_message "ERROR: Missing first chunk name"
   goto failure
) else (
   type null > %TEMP1%
)
if "%TEMP2%" == "" (
   call :print_message "ERROR: Missing second chunk name"
   goto failure
) else (
   type null > %TEMP2%
)
if "%LOGDIR%" == "" (
   call :print_message "ERROR: SAP active log directory does not specified"
   goto failure
)
if not exist "%LOGDIR%\" (
   call :print_message "ERROR: Specified SAP active log directory '%LOGDIR%' does not exist"
   goto failure
)

if "%LOGNAME%" == "" (
   call :print_message "ERROR: Missing SAP logfiles mask (main logfile name) argument"
   goto failure
)

if "%SAP_LOG_API_PATH%" == "" (
   call :print_message "ERROR: Missing SAP NetWeaver logging API jar absolute path argument"
   goto failure
)

if not exist "%SAP_LOG_API_PATH%" (
   call :print_message "ERROR: Specified SAP NetWeaver logging API jar '%SAP_LOG_API_PATH%' does not exist"
   goto failure
)

if "%JAVA_PATH%" == "" (
   call :print_message "WARNING: JRE path is not specified in ES properties. Search for existing Java binaries"
   for /F "tokens=2,3 delims=:" %%A in ('where.exe "%JAVA_EXE%" ^| findstr /N "%JAVA_EXE%" ^| findstr /R "^1:"') do set JAVA_PATH=%%A:%%B
) else (
   set JAVA_PATH=%JAVA_PATH%\%JAVA_EXE%
)

if not exist "%JAVA_PATH%" (
   call :print_message "WARNING: '%JAVA_PATH%' Java binary file does not exist. Default Java binary will be used"
   set JAVA_PATH=%JAVA_EXE%
)
call :print_message "Use Java binary file '%JAVA_PATH%'"

call :print_message "Processing started"
call :print_message "Execute java collect binary '..\bin\getsapjslogs.jar'"

if exist "%CHUNKS_DIR%\" (
   rmdir /S /Q "%CHUNKS_DIR%" >nul 2>>%ES_LOG%
   if exist "%CHUNKS_DIR%\" (
      call :print_message "ERROR: Unable to remove existing temporary directory to copy SAP AS original logfiles '%CHUNKS_DIR%'"
      goto failure
   )
)
mkdir "%CHUNKS_DIR%" >nul 2>>%ES_LOG%
if errorlevel 1 (
   call :print_message "ERROR: Unable to create temporary directory to copy SAP AS original logfiles '%CHUNKS_DIR%'"
   goto failure
)

"%JAVA_PATH%" -Dfile.encoding=UTF-8 -classpath "%JAR_FILE%;%SAP_LOG_API_PATH%;%CLASSPATH%" %JAVA_CLASS% -logdir "%LOGDIR%" -archdir "%ARCHDIR%" -logfile %LOGNAME% -chunk1 %TEMP1% -chunk2 %CHUNKS_DIR% -propsfile %PROPS_FILE% >> %ES_LOG% 2>&1

if errorlevel 1 (
   call :print_message "ERROR: Collect process failed"
   goto failure
)

set ERROR=0
set TARBALL=0
pushd ..\bin
set TSIEM_BIN_DIR=%CD%
popd
set WORK_DIR=%CD%
pushd "%CHUNKS_DIR%"
if exist "*%LOGNAME%*" (
   set TARBALL=1
   call :print_message "Create tarball with SAP AS original logfiles" "%WORK_DIR%\%ES_LOG%"
   "%TSIEM_BIN_DIR%\bsdtar.exe" -cf orig_chunks.tar * >nul 2>>"%WORK_DIR%\%ES_LOG%"
)
popd
if errorlevel 1 (
   call :print_message "ERROR: Failure to create tarball from original logfiles"
   set ERROR=1
)

if %TARBALL% EQU 1 (
   call :print_message "Put tarball with SAP AS original logfiles to second chunk"
   move /Y "%CHUNKS_DIR%\orig_chunks.tar" "%TEMP2%" >nul 2>>%ES_LOG%
   if errorlevel 1 (
      call :print_message "ERROR: Failure to move tarball with original logfiles to second chunk"
      set ERROR=1
   )
)

call :print_message "Free temporary used resources"
rmdir /S /Q "%CHUNKS_DIR%" >nul 2>>%ES_LOG%
if exist "%CHUNKS_DIR%\" (
   call :print_message "ERROR: Unable to remove existing temporary directory to copy SAP AS original logfiles '%CHUNKS_DIR%'"
   goto failure
)

if %ERROR% EQU 1 goto failure

call :print_message "Processing finished [SUCCESS]"
exit /b 0

:print_message

(set OUT_FILE=%~2)
if "%OUT_FILE%" == "" set OUT_FILE=%ES_LOG%
@echo [%date% %time%] %~1.>> "%OUT_FILE%"
exit /b

:failure

call :print_message "Processing finished [FAILURE]"
exit /b 1
